from typing import Optional, Dict, Any
from opexcore.core import RequestBase
from .types import (
    OVPanelToken,
    OVPanelCreateUser,
    OVPanelUpdateUser,
    OVPanelResponseModel,
    OVPanelNodeCreate,
    OVPanelNodeStatus,
    OVPanelSettings,
    OVPanelServerInfo,
)


class OVPanelManager(RequestBase):
    """OVPanel API Manager with all endpoints"""

    @classmethod
    def _generate_headers(cls, token: Optional[str] = None) -> Dict[str, str]:
        """Generate headers for API requests"""
        headers = {"Content-Type": "application/json"}
        if token:
            headers["Authorization"] = f"Bearer {token}"
        return headers

    @classmethod
    async def login(
        cls, host: str, username: str, password: str, timeout: int = 10
    ) -> OVPanelToken:
        """
        Authenticate and obtain an access token.

        :param host: API host URL
        :param username: Admin username
        :param password: Admin password
        :param timeout: Request timeout in seconds
        :return: Token response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/login",
            data={"username": username, "password": password},
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelToken(**data)

    @classmethod
    async def get_all_users(
        cls, host: str, token: str, timeout: int = 10
    ) -> OVPanelResponseModel:
        """
        Get all users in the panel.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Response with list of users
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/user/all",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelResponseModel(**data)

    @classmethod
    async def create_user(
        cls, host: str, token: str, user_data: OVPanelCreateUser, timeout: int = 10
    ) -> OVPanelResponseModel:
        """
        Create a new user.

        :param host: API host URL
        :param token: Authentication token
        :param user_data: User creation data
        :param timeout: Request timeout in seconds
        :return: Response with created user
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/user/create",
            data=user_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelResponseModel(**data)

    @classmethod
    async def update_user(
        cls, host: str, token: str, user_data: OVPanelUpdateUser, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Update an existing user.

        :param host: API host URL
        :param token: Authentication token
        :param user_data: User update data
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/user/update",
            data=user_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def change_user_status(
        cls, host: str, token: str, user_data: OVPanelUpdateUser, timeout: int = 10
    ) -> OVPanelResponseModel:
        """
        Change the status of a user.

        :param host: API host URL
        :param token: Authentication token
        :param user_data: User update data with status
        :param timeout: Request timeout in seconds
        :return: Response with updated user
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/user/change-status",
            data=user_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelResponseModel(**data)

    @classmethod
    async def delete_user(
        cls, host: str, token: str, name: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete a user by name.

        :param host: API host URL
        :param token: Authentication token
        :param name: Username to delete
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/user/delete/{name}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_settings(
        cls, host: str, token: str, timeout: int = 10
    ) -> OVPanelSettings:
        """
        Get panel settings.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Panel settings
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/settings/",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelSettings(**data)

    @classmethod
    async def get_server_info(
        cls, host: str, token: str, timeout: int = 10
    ) -> OVPanelServerInfo:
        """
        Get server information (CPU, memory, etc.).

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Server information
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/settings/server/info",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelServerInfo(**data)

    @classmethod
    async def add_node(
        cls, host: str, token: str, node_data: OVPanelNodeCreate, timeout: int = 10
    ) -> OVPanelResponseModel:
        """
        Add a new node to the panel.

        :param host: API host URL
        :param token: Authentication token
        :param node_data: Node creation data
        :param timeout: Request timeout in seconds
        :return: Response with created node
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/node/add",
            data=node_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelResponseModel(**data)

    @classmethod
    async def update_node(
        cls,
        host: str,
        token: str,
        address: str,
        node_data: OVPanelNodeCreate,
        timeout: int = 10,
    ) -> OVPanelResponseModel:
        """
        Update an existing node.

        :param host: API host URL
        :param token: Authentication token
        :param address: Node address to update
        :param node_data: Node update data
        :param timeout: Request timeout in seconds
        :return: Response with updated node
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/node/update/{address}",
            data=node_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelResponseModel(**data)

    @classmethod
    async def get_node_status(
        cls, host: str, token: str, address: str, timeout: int = 10
    ) -> OVPanelNodeStatus:
        """
        Get the status of a specific node.

        :param host: API host URL
        :param token: Authentication token
        :param address: Node address
        :param timeout: Request timeout in seconds
        :return: Node status response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/node/status/{address}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelNodeStatus(**data)

    @classmethod
    async def list_nodes(
        cls, host: str, token: str, timeout: int = 10
    ) -> OVPanelResponseModel:
        """
        List all nodes in the panel.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Response with list of nodes
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/node/list",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelResponseModel(**data)

    @classmethod
    async def download_ovpn_client(
        cls, host: str, token: str, address: str, name: str, timeout: int = 10
    ) -> bytes:
        """
        Download OVPN client configuration from a node.

        :param host: API host URL
        :param token: Authentication token
        :param address: Node address
        :param name: Client name
        :param timeout: Request timeout in seconds
        :return: OVPN configuration file content
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/node/download/ovpn/{address}/{name}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.read()

    @classmethod
    async def delete_node(
        cls, host: str, token: str, address: str, timeout: int = 10
    ) -> OVPanelResponseModel:
        """
        Delete a node by address.

        :param host: API host URL
        :param token: Authentication token
        :param address: Node address to delete
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/node/delete/{address}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelResponseModel(**data)

    @classmethod
    async def get_all_admins(
        cls, host: str, token: str, timeout: int = 10
    ) -> OVPanelResponseModel:
        """
        Get all admins in the panel.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Response with list of admins
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admin/all",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return OVPanelResponseModel(**data)



from typing import Optional, List
from pydantic import BaseModel, Field


class MarzneshinAdminCreate(BaseModel):
    """Schema for creating a new admin"""

    username: str = Field(..., title="Username")
    is_sudo: bool = Field(..., title="Is Sudo")
    password: str = Field(..., title="Password")
    enabled: bool = Field(True, title="Enabled")
    all_services_access: bool = Field(False, title="All Services Access")
    modify_users_access: bool = Field(True, title="Modify Users Access")
    service_ids: List[int] = Field(default_factory=list, title="Service Ids")
    subscription_url_prefix: str = Field("", title="Subscription Url Prefix")


class MarzneshinAdminPartialModify(BaseModel):
    """Schema for modifying an existing admin"""

    username: Optional[str] = Field(None, title="Username")
    is_sudo: Optional[bool] = Field(None, title="Is Sudo")
    password: Optional[str] = Field(None, title="Password")
    enabled: Optional[bool] = Field(None, title="Enabled")
    all_services_access: Optional[bool] = Field(None, title="All Services Access")
    modify_users_access: Optional[bool] = Field(None, title="Modify Users Access")
    service_ids: Optional[List[int]] = Field(None, title="Service Ids")
    subscription_url_prefix: Optional[str] = Field(
        None, title="Subscription Url Prefix"
    )


class MarzneshinAdmin(BaseModel):
    """Admin response schema"""

    id: int = Field(..., title="Id")
    username: str = Field(..., title="Username")
    is_sudo: bool = Field(..., title="Is Sudo")
    enabled: bool = Field(True, title="Enabled")
    all_services_access: bool = Field(False, title="All Services Access")
    modify_users_access: bool = Field(True, title="Modify Users Access")
    service_ids: List[int] = Field(default_factory=list, title="Service Ids")
    subscription_url_prefix: str = Field("", title="Subscription Url Prefix")


class MarzneshinAdminResponse(BaseModel):
    """Admin response with usage data"""

    id: int = Field(..., title="Id")
    username: str = Field(..., title="Username")
    is_sudo: bool = Field(..., title="Is Sudo")
    enabled: bool = Field(True, title="Enabled")
    all_services_access: bool = Field(False, title="All Services Access")
    modify_users_access: bool = Field(True, title="Modify Users Access")
    service_ids: List[int] = Field(default_factory=list, title="Service Ids")
    subscription_url_prefix: str = Field("", title="Subscription Url Prefix")
    users_data_usage: int = Field(..., title="Users Data Usage")


class MarzneshinToken(BaseModel):
    """Authentication token response"""

    access_token: str = Field(..., title="Access Token")
    token_type: str = Field("bearer", title="Token Type")



from typing import Optional, List, Dict, Any
from datetime import datetime
from opexcore.core import RequestBase
from .types import (
    MarzbanAdmin,
    MarzbanAdminCreate,
    MarzbanAdminModify,
    MarzbanToken,
    MarzbanUserCreate,
    MarzbanUserModify,
    MarzbanUserResponse,
    MarzbanUsersResponse,
    MarzbanUserStatus,
    MarzbanUserUsagesResponse,
    MarzbanUsersUsagesResponse,
    MarzbanNodeCreate,
    MarzbanNodeModify,
    MarzbanNodeResponse,
    MarzbanNodeSettings,
    MarzbanNodesUsageResponse,
    MarzbanCoreStats,
    MarzbanSystemStats,
    MarzbanProxyInbound,
    MarzbanProxyHost,
    MarzbanSubscriptionUserResponse,
    MarzbanUserTemplateCreate,
    MarzbanUserTemplateModify,
    MarzbanUserTemplateResponse,
)


class MarzbanManager(RequestBase):
    """Marzban API Manager with all endpoints"""

    @classmethod
    def _generate_headers(cls, token: Optional[str] = None) -> Dict[str, str]:
        """Generate headers for API requests"""
        headers = {"Content-Type": "application/json"}
        if token:
            headers["Authorization"] = f"Bearer {token}"
        return headers

    @classmethod
    async def admin_token(
        cls, host: str, username: str, password: str, timeout: int = 10
    ) -> MarzbanToken:
        """
        Authenticate an admin and issue a token.

        :param host: API host URL
        :param username: Admin username
        :param password: Admin password
        :param timeout: Request timeout in seconds
        :return: Token response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admin/token",
            data={"username": username, "password": password},
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanToken(**data)

    @classmethod
    async def get_current_admin(
        cls, host: str, token: str, timeout: int = 10
    ) -> MarzbanAdmin:
        """
        Retrieve the current authenticated admin.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admin",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanAdmin(**data)

    @classmethod
    async def create_admin(
        cls, host: str, token: str, admin_data: MarzbanAdminCreate, timeout: int = 10
    ) -> MarzbanAdmin:
        """
        Create a new admin if the current admin has sudo privileges.

        :param host: API host URL
        :param token: Authentication token
        :param admin_data: Admin creation data
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admin",
            data=admin_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanAdmin(**data)

    @classmethod
    async def modify_admin(
        cls,
        host: str,
        token: str,
        username: str,
        admin_data: MarzbanAdminModify,
        timeout: int = 10,
    ) -> MarzbanAdmin:
        """
        Modify an existing admin's details.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param admin_data: Admin modification data
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/admin/{username}",
            data=admin_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanAdmin(**data)

    @classmethod
    async def remove_admin(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove an admin from the database.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/admin/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_admins(
        cls,
        host: str,
        token: str,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        username: Optional[str] = None,
        timeout: int = 10,
    ) -> List[MarzbanAdmin]:
        """
        Fetch a list of admins with optional filters for pagination and username.

        :param host: API host URL
        :param token: Authentication token
        :param offset: Offset for pagination
        :param limit: Limit for pagination
        :param username: Filter by username
        :param timeout: Request timeout in seconds
        :return: List of admin responses
        """
        params = {}
        if offset is not None:
            params["offset"] = offset
        if limit is not None:
            params["limit"] = limit
        if username is not None:
            params["username"] = username

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzbanAdmin(**item) for item in data]

    @classmethod
    async def disable_all_active_users(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Disable all active users under a specific admin.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admin/{username}/users/disable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def activate_all_disabled_users(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Activate all disabled users under a specific admin.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admin/{username}/users/activate",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def reset_admin_usage(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzbanAdmin:
        """
        Resets usage of admin.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admin/usage/reset/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanAdmin(**data)

    @classmethod
    async def get_admin_usage(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> int:
        """
        Retrieve the usage of given admin.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Usage amount
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admin/usage/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_core_stats(
        cls, host: str, token: str, timeout: int = 10
    ) -> MarzbanCoreStats:
        """
        Retrieve core statistics such as version and uptime.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Core stats response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/core",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanCoreStats(**data)

    @classmethod
    async def restart_core(
        cls, host: str, token: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Restart the core and all connected nodes.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/core/restart",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_core_config(
        cls, host: str, token: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Get the current core configuration.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Core configuration
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/core/config",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def modify_core_config(
        cls, host: str, token: str, config: Dict[str, Any], timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Modify the core configuration and restart the core.

        :param host: API host URL
        :param token: Authentication token
        :param config: Core configuration data
        :param timeout: Request timeout in seconds
        :return: Updated core configuration
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/core/config",
            data=config,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_node_settings(
        cls, host: str, token: str, timeout: int = 10
    ) -> MarzbanNodeSettings:
        """
        Retrieve the current node settings, including TLS certificate.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Node settings response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/node/settings",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanNodeSettings(**data)

    @classmethod
    async def add_node(
        cls, host: str, token: str, node_data: MarzbanNodeCreate, timeout: int = 10
    ) -> MarzbanNodeResponse:
        """
        Add a new node to the database and optionally add it as a host.

        :param host: API host URL
        :param token: Authentication token
        :param node_data: Node creation data
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/node",
            data=node_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanNodeResponse(**data)

    @classmethod
    async def get_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> MarzbanNodeResponse:
        """
        Retrieve details of a specific node by its ID.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/node/{node_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanNodeResponse(**data)

    @classmethod
    async def modify_node(
        cls,
        host: str,
        token: str,
        node_id: int,
        node_data: MarzbanNodeModify,
        timeout: int = 10,
    ) -> MarzbanNodeResponse:
        """
        Update a node's details. Only accessible to sudo admins.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param node_data: Node modification data
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/node/{node_id}",
            data=node_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanNodeResponse(**data)

    @classmethod
    async def remove_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete a node and remove it from xray in the background.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/node/{node_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_nodes(
        cls, host: str, token: str, timeout: int = 10
    ) -> List[MarzbanNodeResponse]:
        """
        Retrieve a list of all nodes. Accessible only to sudo admins.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: List of node responses
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzbanNodeResponse(**item) for item in data]

    @classmethod
    async def reconnect_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Trigger a reconnection for the specified node. Only accessible to sudo admins.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/node/{node_id}/reconnect",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_nodes_usage(
        cls,
        host: str,
        token: str,
        start: str = "",
        end: str = "",
        timeout: int = 10,
    ) -> MarzbanNodesUsageResponse:
        """
        Retrieve usage statistics for nodes within a specified date range.

        :param host: API host URL
        :param token: Authentication token
        :param start: Start date (ISO format)
        :param end: End date (ISO format)
        :param timeout: Request timeout in seconds
        :return: Nodes usage response
        """
        params = {"start": start, "end": end}
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes/usage",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanNodesUsageResponse(**data)

    @classmethod
    async def user_subscription(
        cls, host: str, token: str, user_agent: str = "", timeout: int = 10
    ) -> List[str]:
        """
        Provides a subscription link based on the user agent (Clash, V2Ray, etc.).

        :param host: API host URL
        :param token: Subscription token
        :param user_agent: User agent header
        :param timeout: Request timeout in seconds
        :return: Subscription links
        """
        headers = {}
        if user_agent:
            headers["user-agent"] = user_agent

        response = await cls.get(
            url=f"{host.rstrip('/')}/sub/{token}/",
            headers=headers,
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def user_subscription_info(
        cls, host: str, token: str, timeout: int = 10
    ) -> MarzbanSubscriptionUserResponse:
        """
        Retrieves detailed information about the user's subscription.

        :param host: API host URL
        :param token: Subscription token
        :param timeout: Request timeout in seconds
        :return: Subscription user response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/sub/{token}/info",
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanSubscriptionUserResponse(**data)

    @classmethod
    async def user_get_usage(
        cls,
        host: str,
        token: str,
        start: str = "",
        end: str = "",
        timeout: int = 10,
    ) -> Dict[str, Any]:
        """
        Fetches the usage statistics for the user within a specified date range.

        :param host: API host URL
        :param token: Subscription token
        :param start: Start date
        :param end: End date
        :param timeout: Request timeout in seconds
        :return: Usage data
        """
        params = {"start": start, "end": end}
        response = await cls.get(
            url=f"{host.rstrip('/')}/sub/{token}/usage",
            params=params,
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def user_subscription_with_client_type(
        cls,
        host: str,
        token: str,
        client_type: str,
        user_agent: str = "",
        timeout: int = 10,
    ) -> List[str]:
        """
        Provides a subscription link based on the specified client type (e.g., Clash, V2Ray).

        :param host: API host URL
        :param token: Subscription token
        :param client_type: Client type (sing-box, clash-meta, clash, outline, v2ray, v2ray-json)
        :param user_agent: User agent header
        :param timeout: Request timeout in seconds
        :return: Subscription links
        """
        headers = {}
        if user_agent:
            headers["user-agent"] = user_agent

        response = await cls.get(
            url=f"{host.rstrip('/')}/sub/{token}/{client_type}",
            headers=headers,
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_system_stats(
        cls, host: str, token: str, timeout: int = 10
    ) -> MarzbanSystemStats:
        """
        Fetch system stats including memory, CPU, and user metrics.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: System stats response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/system",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanSystemStats(**data)

    @classmethod
    async def get_inbounds(
        cls, host: str, token: str, timeout: int = 10
    ) -> Dict[str, List[MarzbanProxyInbound]]:
        """
        Retrieve inbound configurations grouped by protocol.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Dictionary of inbounds by protocol
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/inbounds",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        result = {}
        for protocol, inbounds in data.items():
            result[protocol] = [MarzbanProxyInbound(**inbound) for inbound in inbounds]
        return result

    @classmethod
    async def get_hosts(
        cls, host: str, token: str, timeout: int = 10
    ) -> Dict[str, List[MarzbanProxyHost]]:
        """
        Get a list of proxy hosts grouped by inbound tag.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Dictionary of hosts by inbound tag
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/hosts",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        result = {}
        for tag, hosts in data.items():
            result[tag] = [MarzbanProxyHost(**host_data) for host_data in hosts]
        return result

    @classmethod
    async def modify_hosts(
        cls,
        host: str,
        token: str,
        modified_hosts: Dict[str, List[MarzbanProxyHost]],
        timeout: int = 10,
    ) -> Dict[str, List[MarzbanProxyHost]]:
        """
        Modify proxy hosts and update the configuration.

        :param host: API host URL
        :param token: Authentication token
        :param modified_hosts: Dictionary of modified hosts by inbound tag
        :param timeout: Request timeout in seconds
        :return: Updated dictionary of hosts
        """
        data = {}
        for tag, hosts in modified_hosts.items():
            data[tag] = [host_obj.dict() for host_obj in hosts]

        response = await cls.put(
            url=f"{host.rstrip('/')}/api/hosts",
            data=data,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        result_data = await response.json()
        result = {}
        for tag, hosts in result_data.items():
            result[tag] = [MarzbanProxyHost(**host_data) for host_data in hosts]
        return result

    @classmethod
    async def add_user_template(
        cls,
        host: str,
        token: str,
        template_data: MarzbanUserTemplateCreate,
        timeout: int = 10,
    ) -> MarzbanUserTemplateResponse:
        """
        Add a new user template.

        :param host: API host URL
        :param token: Authentication token
        :param template_data: User template creation data
        :param timeout: Request timeout in seconds
        :return: User template response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/user_template",
            data=template_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUserTemplateResponse(**data)

    @classmethod
    async def get_user_templates(
        cls,
        host: str,
        token: str,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        timeout: int = 10,
    ) -> List[MarzbanUserTemplateResponse]:
        """
        Get a list of User Templates with optional pagination.

        :param host: API host URL
        :param token: Authentication token
        :param offset: Offset for pagination
        :param limit: Limit for pagination
        :param timeout: Request timeout in seconds
        :return: List of user template responses
        """
        params = {}
        if offset is not None:
            params["offset"] = offset
        if limit is not None:
            params["limit"] = limit

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/user_template",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [MarzbanUserTemplateResponse(**item) for item in data]

    @classmethod
    async def get_user_template(
        cls, host: str, token: str, template_id: int, timeout: int = 10
    ) -> MarzbanUserTemplateResponse:
        """
        Get User Template information by ID.

        :param host: API host URL
        :param token: Authentication token
        :param template_id: Template ID
        :param timeout: Request timeout in seconds
        :return: User template response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/user_template/{template_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUserTemplateResponse(**data)

    @classmethod
    async def modify_user_template(
        cls,
        host: str,
        token: str,
        template_id: int,
        template_data: MarzbanUserTemplateModify,
        timeout: int = 10,
    ) -> MarzbanUserTemplateResponse:
        """
        Modify User Template.

        :param host: API host URL
        :param token: Authentication token
        :param template_id: Template ID
        :param template_data: User template modification data
        :param timeout: Request timeout in seconds
        :return: User template response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/user_template/{template_id}",
            data=template_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUserTemplateResponse(**data)

    @classmethod
    async def remove_user_template(
        cls, host: str, token: str, template_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove a User Template by its ID.

        :param host: API host URL
        :param token: Authentication token
        :param template_id: Template ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/user_template/{template_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def add_user(
        cls, host: str, token: str, user_data: MarzbanUserCreate, timeout: int = 10
    ) -> MarzbanUserResponse:
        """
        Add a new user.

        :param host: API host URL
        :param token: Authentication token
        :param user_data: User creation data
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/user",
            data=user_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUserResponse(**data)

    @classmethod
    async def get_user(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzbanUserResponse:
        """
        Get user information.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/user/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUserResponse(**data)

    @classmethod
    async def modify_user(
        cls,
        host: str,
        token: str,
        username: str,
        user_data: MarzbanUserModify,
        timeout: int = 10,
    ) -> MarzbanUserResponse:
        """
        Modify an existing user.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param user_data: User modification data
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/user/{username}",
            data=user_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUserResponse(**data)

    @classmethod
    async def remove_user(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove a user.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/user/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def reset_user_data_usage(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzbanUserResponse:
        """
        Reset user data usage.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/user/{username}/reset",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUserResponse(**data)

    @classmethod
    async def revoke_user_subscription(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzbanUserResponse:
        """
        Revoke users subscription (Subscription link and proxies).

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/user/{username}/revoke_sub",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUserResponse(**data)

    @classmethod
    async def get_users(
        cls,
        host: str,
        token: str,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        username: Optional[List[str]] = None,
        search: Optional[str] = None,
        admin: Optional[List[str]] = None,
        status: Optional[MarzbanUserStatus] = None,
        sort: Optional[str] = None,
        timeout: int = 10,
    ) -> MarzbanUsersResponse:
        """
        Get all users with optional filters.

        :param host: API host URL
        :param token: Authentication token
        :param offset: Offset for pagination
        :param limit: Limit for pagination
        :param username: Filter by usernames
        :param search: Search query
        :param admin: Filter by admin usernames
        :param status: Filter by status
        :param sort: Sort field
        :param timeout: Request timeout in seconds
        :return: Users response
        """
        params = {}
        if offset is not None:
            params["offset"] = offset
        if limit is not None:
            params["limit"] = limit
        if username is not None:
            params["username"] = username
        if search is not None:
            params["search"] = search
        if admin is not None:
            params["admin"] = admin
        if status is not None:
            params["status"] = status.value
        if sort is not None:
            params["sort"] = sort

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/users",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUsersResponse(**data)

    @classmethod
    async def reset_users_data_usage(
        cls, host: str, token: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Reset all users data usage.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users/reset",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_user_usage(
        cls,
        host: str,
        token: str,
        username: str,
        start: str = "",
        end: str = "",
        timeout: int = 10,
    ) -> MarzbanUserUsagesResponse:
        """
        Get users usage.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param start: Start date
        :param end: End date
        :param timeout: Request timeout in seconds
        :return: User usages response
        """
        params = {"start": start, "end": end}
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/user/{username}/usage",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUserUsagesResponse(**data)

    @classmethod
    async def active_next_plan(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> MarzbanUserResponse:
        """
        Reset user by next plan.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/user/{username}/active-next",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUserResponse(**data)

    @classmethod
    async def get_users_usage(
        cls,
        host: str,
        token: str,
        start: str = "",
        end: str = "",
        admin: Optional[List[str]] = None,
        timeout: int = 10,
    ) -> MarzbanUsersUsagesResponse:
        """
        Get all users usage.

        :param host: API host URL
        :param token: Authentication token
        :param start: Start date
        :param end: End date
        :param admin: Filter by admin usernames
        :param timeout: Request timeout in seconds
        :return: Users usages response
        """
        params = {"start": start, "end": end}
        if admin is not None:
            params["admin"] = admin

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/users/usage",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUsersUsagesResponse(**data)

    @classmethod
    async def set_owner(
        cls,
        host: str,
        token: str,
        username: str,
        admin_username: str,
        timeout: int = 10,
    ) -> MarzbanUserResponse:
        """
        Set a new owner (admin) for a user.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param admin_username: Admin username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/user/{username}/set-owner",
            data={"admin_username": admin_username},
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return MarzbanUserResponse(**data)

    @classmethod
    async def get_expired_users(
        cls,
        host: str,
        token: str,
        expired_after: Optional[datetime] = None,
        expired_before: Optional[datetime] = None,
        timeout: int = 10,
    ) -> List[str]:
        """
        Get users who have expired within the specified date range.

        :param host: API host URL
        :param token: Authentication token
        :param expired_after: Expired after date
        :param expired_before: Expired before date
        :param timeout: Request timeout in seconds
        :return: List of usernames
        """
        params = {}
        if expired_after is not None:
            params["expired_after"] = expired_after.isoformat()
        if expired_before is not None:
            params["expired_before"] = expired_before.isoformat()

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/users/expired",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def delete_expired_users(
        cls,
        host: str,
        token: str,
        expired_after: Optional[datetime] = None,
        expired_before: Optional[datetime] = None,
        timeout: int = 10,
    ) -> List[str]:
        """
        Delete users who have expired within the specified date range.

        :param host: API host URL
        :param token: Authentication token
        :param expired_after: Expired after date
        :param expired_before: Expired before date
        :param timeout: Request timeout in seconds
        :return: List of deleted usernames
        """
        params = {}
        if expired_after is not None:
            params["expired_after"] = expired_after.isoformat()
        if expired_before is not None:
            params["expired_before"] = expired_before.isoformat()

        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/users/expired",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()



from typing import List
from enum import Enum
from pydantic import BaseModel, Field


class MarzneshinConfigTypes(str, Enum):
    """Config types enumeration"""

    LINKS = "links"
    BASE64_LINKS = "base64-links"
    XRAY = "xray"
    SING_BOX = "sing-box"
    CLASH = "clash"
    CLASH_META = "clash-meta"
    TEMPLATE = "template"
    BLOCK = "block"


class MarzneshinSubscriptionRule(BaseModel):
    """Subscription rule"""

    pattern: str = Field(..., title="Pattern", format="regex")
    result: MarzneshinConfigTypes = Field(..., title="Result")


class MarzneshinSubscriptionSettings(BaseModel):
    """Subscription settings"""

    template_on_acceptance: bool = Field(..., title="Template On Acceptance")
    profile_title: str = Field(..., title="Profile Title")
    support_link: str = Field(..., title="Support Link")
    update_interval: int = Field(..., title="Update Interval")
    shuffle_configs: bool = Field(False, title="Shuffle Configs")
    placeholder_if_disabled: bool = Field(True, title="Placeholder If Disabled")
    placeholder_remark: str = Field("disabled", title="Placeholder Remark")
    rules: List[MarzneshinSubscriptionRule] = Field(..., title="Rules")



from typing import Optional
from pydantic import BaseModel, Field


class MarzbanAdminCreate(BaseModel):
    """Schema for creating a new admin"""

    username: str = Field(..., title="Username")
    is_sudo: bool = Field(..., title="Is Sudo")
    password: str = Field(..., title="Password")
    telegram_id: Optional[int] = Field(None, title="Telegram Id")
    discord_webhook: Optional[str] = Field(None, title="Discord Webhook")
    users_usage: Optional[int] = Field(None, title="Users Usage")


class MarzbanAdminModify(BaseModel):
    """Schema for modifying an existing admin"""

    password: Optional[str] = Field(None, title="Password")
    is_sudo: bool = Field(..., title="Is Sudo")
    telegram_id: Optional[int] = Field(None, title="Telegram Id")
    discord_webhook: Optional[str] = Field(None, title="Discord Webhook")


class MarzbanAdmin(BaseModel):
    """Admin response schema"""

    username: str = Field(..., title="Username")
    is_sudo: bool = Field(..., title="Is Sudo")
    telegram_id: Optional[int] = Field(None, title="Telegram Id")
    discord_webhook: Optional[str] = Field(None, title="Discord Webhook")
    users_usage: Optional[int] = Field(None, title="Users Usage")


class MarzbanToken(BaseModel):
    """Authentication token response"""

    access_token: str = Field(..., title="Access Token")
    token_type: str = Field("bearer", title="Token Type")



from typing import Optional, List, Dict, Any
from pydantic import BaseModel, Field


class PasarGuardUserTemplateCreate(BaseModel):
    """Schema for creating a new user template"""

    name: Optional[str] = Field(None, title="Name")
    data_limit: Optional[int] = Field(None, title="Data Limit")
    expire_duration: Optional[int] = Field(None, title="Expire Duration")
    username_prefix: Optional[str] = Field(None, title="Username Prefix", max_length=20)
    username_suffix: Optional[str] = Field(None, title="Username Suffix", max_length=20)
    group_ids: List[int] = Field(..., title="Group Ids")
    extra_settings: Optional[Dict[str, Any]] = Field(None, title="Extra Settings")
    status: Optional[str] = Field(None, title="Status")
    reset_usages: Optional[bool] = Field(None, title="Reset Usages")
    on_hold_timeout: Optional[int] = Field(None, title="On Hold Timeout")
    data_limit_reset_strategy: str = Field(
        "no_reset", title="Data Limit Reset Strategy"
    )
    is_disabled: Optional[bool] = Field(None, title="Is Disabled")


class PasarGuardUserTemplateModify(BaseModel):
    """Schema for modifying an existing user template"""

    name: Optional[str] = Field(None, title="Name")
    data_limit: Optional[int] = Field(None, title="Data Limit")
    expire_duration: Optional[int] = Field(None, title="Expire Duration")
    username_prefix: Optional[str] = Field(None, title="Username Prefix", max_length=20)
    username_suffix: Optional[str] = Field(None, title="Username Suffix", max_length=20)
    group_ids: Optional[List[int]] = Field(None, title="Group Ids")
    extra_settings: Optional[Dict[str, Any]] = Field(None, title="Extra Settings")
    status: Optional[str] = Field(None, title="Status")
    reset_usages: Optional[bool] = Field(None, title="Reset Usages")
    on_hold_timeout: Optional[int] = Field(None, title="On Hold Timeout")
    data_limit_reset_strategy: str = Field(
        "no_reset", title="Data Limit Reset Strategy"
    )
    is_disabled: Optional[bool] = Field(None, title="Is Disabled")


class PasarGuardUserTemplateResponse(PasarGuardUserTemplateCreate):
    """User template response schema"""

    id: int = Field(..., title="Id")



from typing import Optional
from pydantic import BaseModel, Field


class PasarGuardAdminCreate(BaseModel):
    """Schema for creating a new admin"""

    username: str = Field(..., title="Username")
    password: str = Field(..., title="Password")
    is_sudo: bool = Field(..., title="Is Sudo")
    telegram_id: Optional[int] = Field(None, title="Telegram Id")
    discord_webhook: Optional[str] = Field(None, title="Discord Webhook")
    discord_id: Optional[int] = Field(None, title="Discord Id")
    is_disabled: Optional[bool] = Field(None, title="Is Disabled")
    sub_template: Optional[str] = Field(None, title="Sub Template")
    sub_domain: Optional[str] = Field(None, title="Sub Domain")
    profile_title: Optional[str] = Field(None, title="Profile Title")
    support_url: Optional[str] = Field(None, title="Support Url")


class PasarGuardAdminModify(BaseModel):
    """Schema for modifying an existing admin"""

    password: Optional[str] = Field(None, title="Password")
    is_sudo: bool = Field(..., title="Is Sudo")
    telegram_id: Optional[int] = Field(None, title="Telegram Id")
    discord_webhook: Optional[str] = Field(None, title="Discord Webhook")
    discord_id: Optional[int] = Field(None, title="Discord Id")
    is_disabled: Optional[bool] = Field(None, title="Is Disabled")
    sub_template: Optional[str] = Field(None, title="Sub Template")
    sub_domain: Optional[str] = Field(None, title="Sub Domain")
    profile_title: Optional[str] = Field(None, title="Profile Title")
    support_url: Optional[str] = Field(None, title="Support Url")


class PasarGuardAdminDetails(BaseModel):
    """Admin response schema"""

    username: str = Field(..., title="Username")
    telegram_id: Optional[int] = Field(None, title="Telegram Id")
    discord_webhook: Optional[str] = Field(None, title="Discord Webhook")
    sub_domain: Optional[str] = Field(None, title="Sub Domain")
    profile_title: Optional[str] = Field(None, title="Profile Title")
    support_url: Optional[str] = Field(None, title="Support Url")
    id: Optional[int] = Field(None, title="Id")
    is_sudo: bool = Field(..., title="Is Sudo")
    total_users: int = Field(0, title="Total Users")
    used_traffic: int = Field(0, title="Used Traffic")
    is_disabled: bool = Field(False, title="Is Disabled")
    discord_id: Optional[int] = Field(None, title="Discord Id")
    sub_template: Optional[str] = Field(None, title="Sub Template")
    lifetime_used_traffic: Optional[int] = Field(None, title="Lifetime Used Traffic")


class PasarGuardToken(BaseModel):
    """Authentication token response"""

    access_token: str = Field(..., title="Access Token")
    token_type: str = Field("bearer", title="Token Type")



from typing import Optional
from enum import Enum
from pydantic import BaseModel, Field


class RemnawaveUserStatus(str, Enum):
    """User status enumeration"""

    ACTIVE = "ACTIVE"
    DISABLED = "DISABLED"
    LIMITED = "LIMITED"
    EXPIRED = "EXPIRED"


class RemnawaveTrafficLimitStrategy(str, Enum):
    """Traffic limit reset strategy"""

    NO_RESET = "NO_RESET"
    DAY = "DAY"
    WEEK = "WEEK"
    MONTH = "MONTH"


class RemnawaveUser(BaseModel):
    """User response"""

    uuid: str
    short_uuid: str = Field(..., alias="shortUuid")
    username: str
    status: RemnawaveUserStatus
    used_traffic_bytes: int = Field(..., alias="usedTrafficBytes")
    lifetime_used_traffic_bytes: int = Field(..., alias="lifetimeUsedTrafficBytes")
    traffic_limit_bytes: int = Field(..., alias="trafficLimitBytes")
    traffic_limit_strategy: RemnawaveTrafficLimitStrategy = Field(
        ..., alias="trafficLimitStrategy"
    )
    expire_at: str = Field(..., alias="expireAt")
    online_at: Optional[str] = Field(None, alias="onlineAt")
    sub_revoked_at: Optional[str] = Field(None, alias="subRevokedAt")
    trojan_password: str = Field(..., alias="trojanPassword")
    vless_uuid: str = Field(..., alias="vlessUuid")
    ss_password: str = Field(..., alias="ssPassword")
    description: Optional[str] = None
    tag: Optional[str] = None
    telegram_id: Optional[int] = Field(None, alias="telegramId")
    email: Optional[str] = None
    hwid_device_limit: Optional[int] = Field(None, alias="hwidDeviceLimit")
    created_at: str = Field(..., alias="createdAt")
    updated_at: str = Field(..., alias="updatedAt")
    subscription_url: str = Field(..., alias="subscriptionUrl")

    class Config:
        populate_by_name = True
        use_enum_values = True


class RemnawaveUserCreate(BaseModel):
    """User creation data"""

    username: str = Field(..., min_length=3, max_length=32)
    traffic_limit_bytes: int = Field(0, alias="trafficLimitBytes")
    traffic_limit_strategy: RemnawaveTrafficLimitStrategy = Field(
        RemnawaveTrafficLimitStrategy.NO_RESET, alias="trafficLimitStrategy"
    )
    expire_at: str = Field(..., alias="expireAt")
    description: Optional[str] = None
    tag: Optional[str] = None
    telegram_id: Optional[int] = Field(None, alias="telegramId")
    email: Optional[str] = None
    hwid_device_limit: Optional[int] = Field(None, alias="hwidDeviceLimit")

    class Config:
        populate_by_name = True
        use_enum_values = True


class RemnawaveUserUpdate(BaseModel):
    """User update data"""

    traffic_limit_bytes: Optional[int] = Field(None, alias="trafficLimitBytes")
    traffic_limit_strategy: Optional[RemnawaveTrafficLimitStrategy] = Field(
        None, alias="trafficLimitStrategy"
    )
    expire_at: Optional[str] = Field(None, alias="expireAt")
    description: Optional[str] = None
    tag: Optional[str] = None
    telegram_id: Optional[int] = Field(None, alias="telegramId")
    email: Optional[str] = None
    hwid_device_limit: Optional[int] = Field(None, alias="hwidDeviceLimit")

    class Config:
        populate_by_name = True
        use_enum_values = True



from typing import Optional
from pydantic import BaseModel, Field


class RemnawaveToken(BaseModel):
    """Token response"""

    access_token: str = Field(..., alias="accessToken")

    class Config:
        populate_by_name = True


class RemnawaveAdmin(BaseModel):
    """Admin response"""

    uuid: str
    username: str
    telegram_id: Optional[int] = Field(None, alias="telegramId")
    discord_webhook: Optional[str] = Field(None, alias="discordWebhook")
    is_sudo: bool = Field(..., alias="isSudo")
    created_at: str = Field(..., alias="createdAt")
    updated_at: str = Field(..., alias="updatedAt")

    class Config:
        populate_by_name = True


class RemnawaveAdminCreate(BaseModel):
    """Admin creation data"""

    username: str
    password: str
    telegram_id: Optional[int] = Field(None, alias="telegramId")
    discord_webhook: Optional[str] = Field(None, alias="discordWebhook")
    is_sudo: bool = Field(False, alias="isSudo")

    class Config:
        populate_by_name = True


class RemnawaveAdminUpdate(BaseModel):
    """Admin update data"""

    password: Optional[str] = None
    telegram_id: Optional[int] = Field(None, alias="telegramId")
    discord_webhook: Optional[str] = Field(None, alias="discordWebhook")
    is_sudo: Optional[bool] = Field(None, alias="isSudo")

    class Config:
        populate_by_name = True



from typing import Optional, List, Dict, Any
from datetime import datetime
from opexcore.core import RequestBase
from .types import (
    PasarGuardAdminCreate,
    PasarGuardAdminModify,
    PasarGuardAdminDetails,
    PasarGuardToken,
    PasarGuardUserCreate,
    PasarGuardUserModify,
    PasarGuardUserResponse,
    PasarGuardUsersResponse,
    PasarGuardUserStatus,
    PasarGuardSubscriptionUserResponse,
    PasarGuardNodeCreate,
    PasarGuardNodeModify,
    PasarGuardNodeResponse,
    PasarGuardNodeSettings,
    PasarGuardNodeStatus,
    PasarGuardCoreCreate,
    PasarGuardCoreResponse,
    PasarGuardSystemStats,
    PasarGuardGroupCreate,
    PasarGuardGroupModify,
    PasarGuardGroupResponse,
    PasarGuardHostCreate,
    PasarGuardHostResponse,
    PasarGuardUserTemplateCreate,
    PasarGuardUserTemplateModify,
    PasarGuardUserTemplateResponse,
)


class PasarGuardManager(RequestBase):
    """PasarGuard API Manager with all endpoints"""

    @classmethod
    def _generate_headers(cls, token: Optional[str] = None) -> Dict[str, str]:
        """Generate headers for API requests"""
        headers = {"Content-Type": "application/json"}
        if token:
            headers["Authorization"] = f"Bearer {token}"
        return headers

    @classmethod
    async def admin_token(
        cls, host: str, username: str, password: str, timeout: int = 10
    ) -> PasarGuardToken:
        """
        Authenticate an admin and issue a token.

        :param host: API host URL
        :param username: Admin username
        :param password: Admin password
        :param timeout: Request timeout in seconds
        :return: Token response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admin/token",
            data={"username": username, "password": password},
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardToken(**data)

    @classmethod
    async def get_current_admin(
        cls, host: str, token: str, timeout: int = 10
    ) -> PasarGuardAdminDetails:
        """
        Retrieve the current authenticated admin.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admin",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardAdminDetails(**data)

    @classmethod
    async def create_admin(
        cls, host: str, token: str, admin_data: PasarGuardAdminCreate, timeout: int = 10
    ) -> PasarGuardAdminDetails:
        """
        Create a new admin if the current admin has sudo privileges.

        :param host: API host URL
        :param token: Authentication token
        :param admin_data: Admin creation data
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admin",
            data=admin_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardAdminDetails(**data)

    @classmethod
    async def modify_admin(
        cls,
        host: str,
        token: str,
        username: str,
        admin_data: PasarGuardAdminModify,
        timeout: int = 10,
    ) -> PasarGuardAdminDetails:
        """
        Modify an existing admin's details.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param admin_data: Admin modification data
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/admin/{username}",
            data=admin_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardAdminDetails(**data)

    @classmethod
    async def remove_admin(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove an admin from the database.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/admin/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_admins(
        cls,
        host: str,
        token: str,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        username: Optional[str] = None,
        sort: Optional[str] = None,
        timeout: int = 10,
    ) -> List[PasarGuardAdminDetails]:
        """
        Fetch a list of admins with optional filters for pagination and username.

        :param host: API host URL
        :param token: Authentication token
        :param offset: Offset for pagination
        :param limit: Limit for pagination
        :param username: Filter by username
        :param sort: Sort field
        :param timeout: Request timeout in seconds
        :return: List of admin responses
        """
        params = {}
        if offset is not None:
            params["offset"] = offset
        if limit is not None:
            params["limit"] = limit
        if username is not None:
            params["username"] = username
        if sort is not None:
            params["sort"] = sort

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/admins",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [PasarGuardAdminDetails(**item) for item in data]

    @classmethod
    async def reset_admin_usage(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> PasarGuardAdminDetails:
        """
        Resets usage of admin.

        :param host: API host URL
        :param token: Authentication token
        :param username: Admin username
        :param timeout: Request timeout in seconds
        :return: Admin response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/admin/{username}/reset",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardAdminDetails(**data)

    @classmethod
    async def get_system_stats(
        cls,
        host: str,
        token: str,
        admin_username: Optional[str] = None,
        timeout: int = 10,
    ) -> PasarGuardSystemStats:
        """
        Fetch system stats including memory, CPU, and user metrics.

        :param host: API host URL
        :param token: Authentication token
        :param admin_username: Optional admin username filter
        :param timeout: Request timeout in seconds
        :return: System stats response
        """
        params = {}
        if admin_username:
            params["admin_username"] = admin_username

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/system",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardSystemStats(**data)

    @classmethod
    async def get_inbounds(cls, host: str, token: str, timeout: int = 10) -> List[str]:
        """
        Retrieve inbound configurations.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: List of inbound tags
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/inbounds",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def create_group(
        cls, host: str, token: str, group_data: PasarGuardGroupCreate, timeout: int = 10
    ) -> PasarGuardGroupResponse:
        """
        Create a new group.

        :param host: API host URL
        :param token: Authentication token
        :param group_data: Group creation data
        :param timeout: Request timeout in seconds
        :return: Group response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/group",
            data=group_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardGroupResponse(**data)

    @classmethod
    async def get_groups(
        cls,
        host: str,
        token: str,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        timeout: int = 10,
    ) -> List[PasarGuardGroupResponse]:
        """
        List all groups.

        :param host: API host URL
        :param token: Authentication token
        :param offset: Offset for pagination
        :param limit: Limit for pagination
        :param timeout: Request timeout in seconds
        :return: List of group responses
        """
        params = {}
        if offset is not None:
            params["offset"] = offset
        if limit is not None:
            params["limit"] = limit

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/groups",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return (
            [PasarGuardGroupResponse(**item["groups"]) for item in [data]][0]
            if isinstance(data, dict)
            else [PasarGuardGroupResponse(**item) for item in data]
        )

    @classmethod
    async def get_group(
        cls, host: str, token: str, group_id: int, timeout: int = 10
    ) -> PasarGuardGroupResponse:
        """
        Get group details.

        :param host: API host URL
        :param token: Authentication token
        :param group_id: Group ID
        :param timeout: Request timeout in seconds
        :return: Group response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/group/{group_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardGroupResponse(**data)

    @classmethod
    async def modify_group(
        cls,
        host: str,
        token: str,
        group_id: int,
        group_data: PasarGuardGroupModify,
        timeout: int = 10,
    ) -> PasarGuardGroupResponse:
        """
        Modify group.

        :param host: API host URL
        :param token: Authentication token
        :param group_id: Group ID
        :param group_data: Group modification data
        :param timeout: Request timeout in seconds
        :return: Group response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/group/{group_id}",
            data=group_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardGroupResponse(**data)

    @classmethod
    async def remove_group(
        cls, host: str, token: str, group_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove group.

        :param host: API host URL
        :param token: Authentication token
        :param group_id: Group ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/group/{group_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def create_core(
        cls, host: str, token: str, core_data: PasarGuardCoreCreate, timeout: int = 10
    ) -> PasarGuardCoreResponse:
        """
        Create a new core configuration.

        :param host: API host URL
        :param token: Authentication token
        :param core_data: Core creation data
        :param timeout: Request timeout in seconds
        :return: Core response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/core",
            data=core_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardCoreResponse(**data)

    @classmethod
    async def get_core(
        cls, host: str, token: str, core_id: int, timeout: int = 10
    ) -> PasarGuardCoreResponse:
        """
        Get a core configuration by its ID.

        :param host: API host URL
        :param token: Authentication token
        :param core_id: Core ID
        :param timeout: Request timeout in seconds
        :return: Core response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/core/{core_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardCoreResponse(**data)

    @classmethod
    async def modify_core(
        cls,
        host: str,
        token: str,
        core_id: int,
        core_data: PasarGuardCoreCreate,
        restart_nodes: bool = False,
        timeout: int = 10,
    ) -> PasarGuardCoreResponse:
        """
        Update an existing core configuration.

        :param host: API host URL
        :param token: Authentication token
        :param core_id: Core ID
        :param core_data: Core modification data
        :param restart_nodes: Whether to restart nodes
        :param timeout: Request timeout in seconds
        :return: Core response
        """
        params = {"restart_nodes": restart_nodes}
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/core/{core_id}",
            params=params,
            data=core_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardCoreResponse(**data)

    @classmethod
    async def delete_core(
        cls,
        host: str,
        token: str,
        core_id: int,
        restart_nodes: bool = False,
        timeout: int = 10,
    ) -> Dict[str, Any]:
        """
        Delete a core configuration.

        :param host: API host URL
        :param token: Authentication token
        :param core_id: Core ID
        :param restart_nodes: Whether to restart nodes
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        params = {"restart_nodes": restart_nodes}
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/core/{core_id}",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_cores(
        cls,
        host: str,
        token: str,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        timeout: int = 10,
    ) -> List[PasarGuardCoreResponse]:
        """
        Get a list of all core configurations.

        :param host: API host URL
        :param token: Authentication token
        :param offset: Offset for pagination
        :param limit: Limit for pagination
        :param timeout: Request timeout in seconds
        :return: List of core responses
        """
        params = {}
        if offset is not None:
            params["offset"] = offset
        if limit is not None:
            params["limit"] = limit

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/cores",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        cores = data.get("cores", []) if isinstance(data, dict) else data
        return [PasarGuardCoreResponse(**item) for item in cores]

    @classmethod
    async def create_host(
        cls, host: str, token: str, host_data: PasarGuardHostCreate, timeout: int = 10
    ) -> PasarGuardHostResponse:
        """
        Create a new host.

        :param host: API host URL
        :param token: Authentication token
        :param host_data: Host creation data
        :param timeout: Request timeout in seconds
        :return: Host response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/host/",
            data=host_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardHostResponse(**data)

    @classmethod
    async def get_host(
        cls, host: str, token: str, host_id: int, timeout: int = 10
    ) -> PasarGuardHostResponse:
        """
        Get host by ID.

        :param host: API host URL
        :param token: Authentication token
        :param host_id: Host ID
        :param timeout: Request timeout in seconds
        :return: Host response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/host/{host_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardHostResponse(**data)

    @classmethod
    async def modify_host(
        cls,
        host: str,
        token: str,
        host_id: int,
        host_data: PasarGuardHostCreate,
        timeout: int = 10,
    ) -> PasarGuardHostResponse:
        """
        Modify host by ID.

        :param host: API host URL
        :param token: Authentication token
        :param host_id: Host ID
        :param host_data: Host modification data
        :param timeout: Request timeout in seconds
        :return: Host response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/host/{host_id}",
            data=host_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardHostResponse(**data)

    @classmethod
    async def remove_host(
        cls, host: str, token: str, host_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove host by ID.

        :param host: API host URL
        :param token: Authentication token
        :param host_id: Host ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/host/{host_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_hosts(
        cls,
        host: str,
        token: str,
        offset: int = 0,
        limit: int = 0,
        timeout: int = 10,
    ) -> List[PasarGuardHostResponse]:
        """
        Get proxy hosts.

        :param host: API host URL
        :param token: Authentication token
        :param offset: Offset for pagination
        :param limit: Limit for pagination
        :param timeout: Request timeout in seconds
        :return: List of host responses
        """
        params = {"offset": offset, "limit": limit}
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/hosts",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [PasarGuardHostResponse(**item) for item in data]

    @classmethod
    async def get_node_settings(
        cls, host: str, token: str, timeout: int = 10
    ) -> PasarGuardNodeSettings:
        """
        Retrieve the current node settings.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: Node settings response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/node/settings",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardNodeSettings(**data)

    @classmethod
    async def get_nodes(
        cls,
        host: str,
        token: str,
        core_id: Optional[int] = None,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        status: Optional[List[PasarGuardNodeStatus]] = None,
        enabled: bool = False,
        ids: Optional[List[int]] = None,
        search: Optional[str] = None,
        timeout: int = 10,
    ) -> List[PasarGuardNodeResponse]:
        """
        Retrieve a list of all nodes.

        :param host: API host URL
        :param token: Authentication token
        :param core_id: Filter by core ID
        :param offset: Offset for pagination
        :param limit: Limit for pagination
        :param status: Filter by status list
        :param enabled: Filter by enabled status
        :param ids: Filter by node IDs
        :param search: Search query
        :param timeout: Request timeout in seconds
        :return: List of node responses
        """
        params = {"enabled": enabled}
        if core_id is not None:
            params["core_id"] = core_id
        if offset is not None:
            params["offset"] = offset
        if limit is not None:
            params["limit"] = limit
        if status is not None:
            params["status"] = [s.value for s in status]
        if ids is not None:
            params["ids"] = ids
        if search is not None:
            params["search"] = search

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [PasarGuardNodeResponse(**item) for item in data]

    @classmethod
    async def create_node(
        cls, host: str, token: str, node_data: PasarGuardNodeCreate, timeout: int = 10
    ) -> PasarGuardNodeResponse:
        """
        Create a new node.

        :param host: API host URL
        :param token: Authentication token
        :param node_data: Node creation data
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/node",
            data=node_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardNodeResponse(**data)

    @classmethod
    async def get_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> PasarGuardNodeResponse:
        """
        Retrieve details of a specific node by its ID.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/node/{node_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardNodeResponse(**data)

    @classmethod
    async def modify_node(
        cls,
        host: str,
        token: str,
        node_id: int,
        node_data: PasarGuardNodeModify,
        timeout: int = 10,
    ) -> PasarGuardNodeResponse:
        """
        Modify a node's details.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param node_data: Node modification data
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/node/{node_id}",
            data=node_data.dict(exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardNodeResponse(**data)

    @classmethod
    async def remove_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove a node.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/node/{node_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def reset_node_usage(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> PasarGuardNodeResponse:
        """
        Reset node traffic usage.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/node/{node_id}/reset",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardNodeResponse(**data)

    @classmethod
    async def reconnect_node(
        cls, host: str, token: str, node_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Trigger a reconnection for the specified node.

        :param host: API host URL
        :param token: Authentication token
        :param node_id: Node ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/node/{node_id}/reconnect",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def create_user_template(
        cls,
        host: str,
        token: str,
        template_data: PasarGuardUserTemplateCreate,
        timeout: int = 10,
    ) -> PasarGuardUserTemplateResponse:
        """
        Create a new user template.

        :param host: API host URL
        :param token: Authentication token
        :param template_data: User template creation data
        :param timeout: Request timeout in seconds
        :return: User template response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/user_template",
            data=template_data.dict(),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardUserTemplateResponse(**data)

    @classmethod
    async def get_user_template(
        cls, host: str, token: str, template_id: int, timeout: int = 10
    ) -> PasarGuardUserTemplateResponse:
        """
        Get User Template information with id.

        :param host: API host URL
        :param token: Authentication token
        :param template_id: Template ID
        :param timeout: Request timeout in seconds
        :return: User template response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/user_template/{template_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardUserTemplateResponse(**data)

    @classmethod
    async def modify_user_template(
        cls,
        host: str,
        token: str,
        template_id: int,
        template_data: PasarGuardUserTemplateModify,
        timeout: int = 10,
    ) -> PasarGuardUserTemplateResponse:
        """
        Modify User Template.

        :param host: API host URL
        :param token: Authentication token
        :param template_id: Template ID
        :param template_data: User template modification data
        :param timeout: Request timeout in seconds
        :return: User template response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/user_template/{template_id}",
            data=template_data.dict(exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardUserTemplateResponse(**data)

    @classmethod
    async def remove_user_template(
        cls, host: str, token: str, template_id: int, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove a User Template by its ID.

        :param host: API host URL
        :param token: Authentication token
        :param template_id: Template ID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/user_template/{template_id}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_user_templates(
        cls,
        host: str,
        token: str,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        timeout: int = 10,
    ) -> List[PasarGuardUserTemplateResponse]:
        """
        Get a list of User Templates with optional pagination.

        :param host: API host URL
        :param token: Authentication token
        :param offset: Offset for pagination
        :param limit: Limit for pagination
        :param timeout: Request timeout in seconds
        :return: List of user template responses
        """
        params = {}
        if offset is not None:
            params["offset"] = offset
        if limit is not None:
            params["limit"] = limit

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/user_templates",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return [PasarGuardUserTemplateResponse(**item) for item in data]

    @classmethod
    async def create_user(
        cls, host: str, token: str, user_data: PasarGuardUserCreate, timeout: int = 10
    ) -> PasarGuardUserResponse:
        """
        Create a new user.

        :param host: API host URL
        :param token: Authentication token
        :param user_data: User creation data
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/user",
            data=user_data.dict(exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardUserResponse(**data)

    @classmethod
    async def get_user(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> PasarGuardUserResponse:
        """
        Get user information.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/user/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardUserResponse(**data)

    @classmethod
    async def modify_user(
        cls,
        host: str,
        token: str,
        username: str,
        user_data: PasarGuardUserModify,
        timeout: int = 10,
    ) -> PasarGuardUserResponse:
        """
        Modify an existing user.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param user_data: User modification data
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/user/{username}",
            data=user_data.dict(exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardUserResponse(**data)

    @classmethod
    async def remove_user(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Remove a user.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/user/{username}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def reset_user_data_usage(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> PasarGuardUserResponse:
        """
        Reset user data usage.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/user/{username}/reset",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardUserResponse(**data)

    @classmethod
    async def revoke_user_subscription(
        cls, host: str, token: str, username: str, timeout: int = 10
    ) -> PasarGuardUserResponse:
        """
        Revoke users subscription (Subscription link and proxies).

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/user/{username}/revoke_sub",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardUserResponse(**data)

    @classmethod
    async def get_users(
        cls,
        host: str,
        token: str,
        offset: Optional[int] = None,
        limit: Optional[int] = None,
        username: Optional[List[str]] = None,
        admin: Optional[List[str]] = None,
        group: Optional[List[int]] = None,
        search: Optional[str] = None,
        status: Optional[PasarGuardUserStatus] = None,
        sort: Optional[str] = None,
        proxy_id: Optional[str] = None,
        load_sub: bool = False,
        timeout: int = 10,
    ) -> PasarGuardUsersResponse:
        """
        Get all users.

        :param host: API host URL
        :param token: Authentication token
        :param offset: Offset for pagination
        :param limit: Limit for pagination
        :param username: Filter by usernames
        :param admin: Filter by admin usernames
        :param group: Filter by group IDs
        :param search: Search query
        :param status: Filter by status
        :param sort: Sort field
        :param proxy_id: Filter by proxy ID
        :param load_sub: Load subscription data
        :param timeout: Request timeout in seconds
        :return: Users response
        """
        params = {"load_sub": load_sub}
        if offset is not None:
            params["offset"] = offset
        if limit is not None:
            params["limit"] = limit
        if username is not None:
            params["username"] = username
        if admin is not None:
            params["admin"] = admin
        if group is not None:
            params["group"] = group
        if search is not None:
            params["search"] = search
        if status is not None:
            params["status"] = status.value
        if sort is not None:
            params["sort"] = sort
        if proxy_id is not None:
            params["proxy_id"] = proxy_id

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/users",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardUsersResponse(**data)

    @classmethod
    async def set_user_owner(
        cls,
        host: str,
        token: str,
        username: str,
        admin_username: str,
        timeout: int = 10,
    ) -> PasarGuardUserResponse:
        """
        Set a new owner (admin) for a user.

        :param host: API host URL
        :param token: Authentication token
        :param username: Username
        :param admin_username: Admin username
        :param timeout: Request timeout in seconds
        :return: User response
        """
        params = {"admin_username": admin_username}
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/user/{username}/set_owner",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardUserResponse(**data)

    @classmethod
    async def get_expired_users(
        cls,
        host: str,
        token: str,
        admin_username: Optional[str] = None,
        expired_after: Optional[datetime] = None,
        expired_before: Optional[datetime] = None,
        timeout: int = 10,
    ) -> List[str]:
        """
        Get users who have expired within the specified date range.

        :param host: API host URL
        :param token: Authentication token
        :param admin_username: Filter by admin username
        :param expired_after: Expired after date
        :param expired_before: Expired before date
        :param timeout: Request timeout in seconds
        :return: List of usernames
        """
        params = {}
        if admin_username:
            params["admin_username"] = admin_username
        if expired_after is not None:
            params["expired_after"] = expired_after.isoformat()
        if expired_before is not None:
            params["expired_before"] = expired_before.isoformat()

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/users/expired",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def delete_expired_users(
        cls,
        host: str,
        token: str,
        admin_username: Optional[str] = None,
        expired_after: Optional[datetime] = None,
        expired_before: Optional[datetime] = None,
        timeout: int = 10,
    ) -> Dict[str, Any]:
        """
        Delete users who have expired within the specified date range.

        :param host: API host URL
        :param token: Authentication token
        :param admin_username: Filter by admin username
        :param expired_after: Expired after date
        :param expired_before: Expired before date
        :param timeout: Request timeout in seconds
        :return: Response data with deleted users info
        """
        params = {}
        if admin_username:
            params["admin_username"] = admin_username
        if expired_after is not None:
            params["expired_after"] = expired_after.isoformat()
        if expired_before is not None:
            params["expired_before"] = expired_before.isoformat()

        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/users/expired",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def user_subscription(
        cls, host: str, token: str, user_agent: str = "", timeout: int = 10
    ) -> List[str]:
        """
        Provides a subscription link based on the user agent.

        :param host: API host URL
        :param token: Subscription token
        :param user_agent: User agent header
        :param timeout: Request timeout in seconds
        :return: Subscription links
        """
        headers = {}
        if user_agent:
            headers["user-agent"] = user_agent

        response = await cls.get(
            url=f"{host.rstrip('/')}/sub/{token}/",
            headers=headers,
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def user_subscription_info(
        cls, host: str, token: str, timeout: int = 10
    ) -> PasarGuardSubscriptionUserResponse:
        """
        Retrieves detailed information about the user's subscription.

        :param host: API host URL
        :param token: Subscription token
        :param timeout: Request timeout in seconds
        :return: Subscription user response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/sub/{token}/info",
            timeout=timeout,
        )
        data = await response.json()
        return PasarGuardSubscriptionUserResponse(**data)

    @classmethod
    async def user_subscription_with_client_type(
        cls,
        host: str,
        token: str,
        client_type: str,
        timeout: int = 10,
    ) -> List[str]:
        """
        Provides a subscription link based on the specified client type.

        :param host: API host URL
        :param token: Subscription token
        :param client_type: Client type (links, links_base64, xray, sing_box, clash, clash_meta, outline, block)
        :param timeout: Request timeout in seconds
        :return: Subscription links
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/sub/{token}/{client_type}",
            timeout=timeout,
        )
        return await response.json()



from typing import Optional, Dict, List, Any
from datetime import datetime
from enum import Enum
from pydantic import BaseModel, Field


class MarzbanUserStatus(str, Enum):
    """User status enumeration"""

    ACTIVE = "active"
    DISABLED = "disabled"
    LIMITED = "limited"
    EXPIRED = "expired"
    ON_HOLD = "on_hold"


class MarzbanUserStatusCreate(str, Enum):
    """User status for creation"""

    ACTIVE = "active"
    ON_HOLD = "on_hold"


class MarzbanUserStatusModify(str, Enum):
    """User status for modification"""

    ACTIVE = "active"
    DISABLED = "disabled"
    ON_HOLD = "on_hold"


class MarzbanUserDataLimitResetStrategy(str, Enum):
    """Data limit reset strategy"""

    NO_RESET = "no_reset"
    DAY = "day"
    WEEK = "week"
    MONTH = "month"
    YEAR = "year"


class MarzbanProxyTypes(str, Enum):
    """Proxy types enumeration"""

    VMESS = "vmess"
    VLESS = "vless"
    TROJAN = "trojan"
    SHADOWSOCKS = "shadowsocks"


class MarzbanNextPlanModel(BaseModel):
    """Next plan model for user"""

    data_limit: Optional[int] = Field(None, title="Data Limit")
    expire: Optional[int] = Field(None, title="Expire")
    add_remaining_traffic: bool = Field(False, title="Add Remaining Traffic")
    fire_on_either: bool = Field(True, title="Fire On Either")


class MarzbanUserCreate(BaseModel):
    """Schema for creating a new user"""

    username: str = Field(..., title="Username")
    status: Optional[MarzbanUserStatusCreate] = Field(None, title="Status")
    proxies: Dict[MarzbanProxyTypes, Dict[str, str]] = Field(
        default_factory=dict, title="Proxies"
    )
    expire: Optional[int] = Field(None, title="Expire", description="UTC timestamp")
    data_limit: Optional[int] = Field(
        None, title="Data Limit", description="In bytes, 0 means unlimited"
    )
    data_limit_reset_strategy: MarzbanUserDataLimitResetStrategy = Field(
        MarzbanUserDataLimitResetStrategy.NO_RESET, title="Data Limit Reset Strategy"
    )
    inbounds: Dict[MarzbanProxyTypes, List[str]] = Field(
        default_factory=dict, title="Inbounds"
    )
    note: Optional[str] = Field(None, title="Note")
    sub_updated_at: Optional[datetime] = Field(None, title="Sub Updated At")
    sub_last_user_agent: Optional[str] = Field(None, title="Sub Last User Agent")
    online_at: Optional[datetime] = Field(None, title="Online At")
    on_hold_expire_duration: Optional[int] = Field(
        None, title="On Hold Expire Duration"
    )
    on_hold_timeout: Optional[datetime] = Field(None, title="On Hold Timeout")
    auto_delete_in_days: Optional[int] = Field(None, title="Auto Delete In Days")
    next_plan: Optional[MarzbanNextPlanModel] = Field(None, title="Next Plan")


class MarzbanUserModify(BaseModel):
    """Schema for modifying an existing user"""

    status: Optional[MarzbanUserStatusModify] = Field(None, title="Status")
    proxies: Dict[MarzbanProxyTypes, Dict[str, str]] = Field(
        default_factory=dict, title="Proxies"
    )
    expire: Optional[int] = Field(None, title="Expire")
    data_limit: Optional[int] = Field(None, title="Data Limit")
    data_limit_reset_strategy: Optional[MarzbanUserDataLimitResetStrategy] = Field(
        None, title="Data Limit Reset Strategy"
    )
    inbounds: Dict[MarzbanProxyTypes, List[str]] = Field(
        default_factory=dict, title="Inbounds"
    )
    note: Optional[str] = Field(None, title="Note")
    sub_updated_at: Optional[datetime] = Field(None, title="Sub Updated At")
    sub_last_user_agent: Optional[str] = Field(None, title="Sub Last User Agent")
    online_at: Optional[datetime] = Field(None, title="Online At")
    on_hold_expire_duration: Optional[int] = Field(
        None, title="On Hold Expire Duration"
    )
    on_hold_timeout: Optional[datetime] = Field(None, title="On Hold Timeout")
    auto_delete_in_days: Optional[int] = Field(None, title="Auto Delete In Days")
    next_plan: Optional[MarzbanNextPlanModel] = Field(None, title="Next Plan")


class MarzbanUserResponse(BaseModel):
    """User response schema"""

    username: str = Field(..., title="Username")
    status: MarzbanUserStatus = Field(..., title="Status")
    proxies: Dict[str, Any] = Field(..., title="Proxies")
    expire: Optional[int] = Field(None, title="Expire")
    data_limit: Optional[int] = Field(None, title="Data Limit")
    data_limit_reset_strategy: MarzbanUserDataLimitResetStrategy = Field(
        MarzbanUserDataLimitResetStrategy.NO_RESET, title="Data Limit Reset Strategy"
    )
    inbounds: Dict[MarzbanProxyTypes, List[str]] = Field(
        default_factory=dict, title="Inbounds"
    )
    note: Optional[str] = Field(None, title="Note")
    sub_updated_at: Optional[datetime] = Field(None, title="Sub Updated At")
    sub_last_user_agent: Optional[str] = Field(None, title="Sub Last User Agent")
    online_at: Optional[datetime] = Field(None, title="Online At")
    on_hold_expire_duration: Optional[int] = Field(
        None, title="On Hold Expire Duration"
    )
    on_hold_timeout: Optional[datetime] = Field(None, title="On Hold Timeout")
    auto_delete_in_days: Optional[int] = Field(None, title="Auto Delete In Days")
    next_plan: Optional[MarzbanNextPlanModel] = Field(None, title="Next Plan")
    used_traffic: int = Field(..., title="Used Traffic")
    lifetime_used_traffic: int = Field(0, title="Lifetime Used Traffic")
    created_at: datetime = Field(..., title="Created At")
    links: List[str] = Field(default_factory=list, title="Links")
    subscription_url: str = Field("", title="Subscription Url")
    excluded_inbounds: Dict[MarzbanProxyTypes, List[str]] = Field(
        default_factory=dict, title="Excluded Inbounds"
    )
    admin: Optional[Any] = Field(None, title="Admin")


class MarzbanUsersResponse(BaseModel):
    """Multiple users response schema"""

    users: List[MarzbanUserResponse] = Field(..., title="Users")
    total: int = Field(..., title="Total")


class MarzbanUserUsageResponse(BaseModel):
    """User usage response schema"""

    node_id: Optional[int] = Field(None, title="Node Id")
    node_name: str = Field(..., title="Node Name")
    used_traffic: int = Field(..., title="Used Traffic")


class MarzbanUserUsagesResponse(BaseModel):
    """User usages response schema"""

    username: str = Field(..., title="Username")
    usages: List[MarzbanUserUsageResponse] = Field(..., title="Usages")


class MarzbanUsersUsagesResponse(BaseModel):
    """Multiple users usages response schema"""

    usages: List[MarzbanUserUsageResponse] = Field(..., title="Usages")



from .manager import OVPanelManager
from .types import (
    OVPanelToken,
    OVPanelAdmin,
    OVPanelUser,
    OVPanelCreateUser,
    OVPanelUpdateUser,
    OVPanelResponseModel,
    OVPanelNode,
    OVPanelNodeCreate,
    OVPanelNodeStatus,
    OVPanelSettings,
    OVPanelServerInfo,
)

__all__ = [
    "OVPanelManager",
    "OVPanelToken",
    "OVPanelAdmin",
    "OVPanelUser",
    "OVPanelCreateUser",
    "OVPanelUpdateUser",
    "OVPanelResponseModel",
    "OVPanelNode",
    "OVPanelNodeCreate",
    "OVPanelNodeStatus",
    "OVPanelSettings",
    "OVPanelServerInfo",
]



from typing import Optional, Dict, Any, List
from pydantic import BaseModel, Field
from datetime import datetime


class PasarGuardCoreCreate(BaseModel):
    """Schema for creating a new core config"""

    name: Optional[str] = Field(None, title="Name")
    config: Dict[str, Any] = Field(..., title="Config")
    exclude_inbound_tags: Optional[List[str]] = Field(
        None, title="Exclude Inbound Tags"
    )
    fallbacks_inbound_tags: Optional[List[str]] = Field(
        None, title="Fallbacks Inbound Tags"
    )


class PasarGuardCoreResponse(BaseModel):
    """Core response schema"""

    id: int = Field(..., title="Id")
    name: str = Field(..., title="Name")
    config: Dict[str, Any] = Field(..., title="Config")
    exclude_inbound_tags: List[str] = Field(..., title="Exclude Inbound Tags")
    fallbacks_inbound_tags: List[str] = Field(..., title="Fallbacks Inbound Tags")
    created_at: datetime = Field(..., title="Created At")



from typing import Optional, List, Dict, Any
from opexcore.core import RequestBase
from .types import (
    RemnawaveToken,
    RemnawaveUser,
    RemnawaveUserCreate,
    RemnawaveUserUpdate,
    RemnawaveNode,
    RemnawaveNodeCreate,
    RemnawaveNodeUpdate,
    RemnawaveHost,
    RemnawaveHostCreate,
    RemnawaveHostUpdate,
    RemnawaveSubscription,
    RemnawaveSystemStats,
)


class RemnawaveManager(RequestBase):
    """Remnawave API Manager with all endpoints"""

    @classmethod
    def _generate_headers(cls, token: Optional[str] = None) -> Dict[str, str]:
        """Generate headers for API requests"""
        headers = {"Content-Type": "application/json"}
        if token:
            headers["Authorization"] = f"Bearer {token}"
        return headers

    @classmethod
    async def admin_login(
        cls, host: str, username: str, password: str, timeout: int = 10
    ) -> RemnawaveToken:
        """
        Login as admin and receive access token.

        :param host: API host URL
        :param username: Admin username
        :param password: Admin password
        :param timeout: Request timeout in seconds
        :return: Token response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/auth/login",
            data={"username": username, "password": password},
            headers={"Content-Type": "application/json"},
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveToken(**data.get("response", data))

    @classmethod
    async def admin_register(
        cls, host: str, username: str, password: str, timeout: int = 10
    ) -> RemnawaveToken:
        """
        Register as superadmin.

        :param host: API host URL
        :param username: Admin username
        :param password: Admin password
        :param timeout: Request timeout in seconds
        :return: Token response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/auth/register",
            data={"username": username, "password": password},
            headers={"Content-Type": "application/json"},
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveToken(**data.get("response", data))

    @classmethod
    async def create_user(
        cls, host: str, token: str, user_data: RemnawaveUserCreate, timeout: int = 10
    ) -> RemnawaveUser:
        """
        Create a new user.

        :param host: API host URL
        :param token: Authentication token
        :param user_data: User creation data
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users",
            data=user_data.dict(by_alias=True, exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveUser(**data.get("response", data))

    @classmethod
    async def get_users(
        cls,
        host: str,
        token: str,
        size: int = 25,
        start: int = 0,
        sort: Optional[str] = None,
        timeout: int = 10,
    ) -> List[RemnawaveUser]:
        """
        Get all users with pagination.

        :param host: API host URL
        :param token: Authentication token
        :param size: Number of users to return
        :param start: Start index (offset)
        :param sort: Sort field
        :param timeout: Request timeout in seconds
        :return: List of users
        """
        params = {"size": size, "start": start}
        if sort:
            params["sort"] = sort

        response = await cls.get(
            url=f"{host.rstrip('/')}/api/users",
            params=params,
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        users_data = data.get("response", {}).get("users", [])
        return [RemnawaveUser(**user) for user in users_data]

    @classmethod
    async def get_user(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> RemnawaveUser:
        """
        Get user by UUID.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: User UUID
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/users/{uuid}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveUser(**data.get("response", data))

    @classmethod
    async def update_user(
        cls,
        host: str,
        token: str,
        uuid: str,
        user_data: RemnawaveUserUpdate,
        timeout: int = 10,
    ) -> RemnawaveUser:
        """
        Update user by UUID.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: User UUID
        :param user_data: User update data
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/users/{uuid}",
            data=user_data.dict(by_alias=True, exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveUser(**data.get("response", data))

    @classmethod
    async def delete_user(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete user by UUID.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: User UUID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/users/{uuid}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def disable_user(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> RemnawaveUser:
        """
        Disable a user.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: User UUID
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users/{uuid}/actions/disable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveUser(**data.get("response", data))

    @classmethod
    async def enable_user(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> RemnawaveUser:
        """
        Enable a user.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: User UUID
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users/{uuid}/actions/enable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveUser(**data.get("response", data))

    @classmethod
    async def reset_user_traffic(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> RemnawaveUser:
        """
        Reset user traffic.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: User UUID
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users/{uuid}/actions/reset-traffic",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveUser(**data.get("response", data))

    @classmethod
    async def revoke_user_subscription(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> RemnawaveUser:
        """
        Revoke user subscription.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: User UUID
        :param timeout: Request timeout in seconds
        :return: User response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/users/{uuid}/actions/revoke",
            headers=cls._generate_headers(token),
            timeout=timeout,
            data={},
        )
        data = await response.json()
        return RemnawaveUser(**data.get("response", data))

    @classmethod
    async def get_nodes(
        cls, host: str, token: str, timeout: int = 10
    ) -> List[RemnawaveNode]:
        """
        Get all nodes.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: List of nodes
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        nodes_data = data.get("response", [])
        return [RemnawaveNode(**node) for node in nodes_data]

    @classmethod
    async def create_node(
        cls, host: str, token: str, node_data: RemnawaveNodeCreate, timeout: int = 10
    ) -> RemnawaveNode:
        """
        Create a new node.

        :param host: API host URL
        :param token: Authentication token
        :param node_data: Node creation data
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/nodes",
            data=node_data.dict(by_alias=True, exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveNode(**data.get("response", data))

    @classmethod
    async def get_node(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> RemnawaveNode:
        """
        Get node by UUID.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: Node UUID
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/nodes/{uuid}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveNode(**data.get("response", data))

    @classmethod
    async def update_node(
        cls,
        host: str,
        token: str,
        uuid: str,
        node_data: RemnawaveNodeUpdate,
        timeout: int = 10,
    ) -> RemnawaveNode:
        """
        Update node by UUID.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: Node UUID
        :param node_data: Node update data
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/nodes/{uuid}",
            data=node_data.dict(by_alias=True, exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveNode(**data.get("response", data))

    @classmethod
    async def delete_node(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete a node.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: Node UUID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/nodes/{uuid}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def enable_node(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> RemnawaveNode:
        """
        Enable a node.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: Node UUID
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/nodes/{uuid}/actions/enable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveNode(**data.get("response", data))

    @classmethod
    async def disable_node(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> RemnawaveNode:
        """
        Disable a node.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: Node UUID
        :param timeout: Request timeout in seconds
        :return: Node response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/nodes/{uuid}/actions/disable",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveNode(**data.get("response", data))

    @classmethod
    async def restart_node(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Restart node.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: Node UUID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/nodes/{uuid}/actions/restart",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def reset_node_traffic(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Reset node traffic.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: Node UUID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/nodes/{uuid}/actions/reset-traffic",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_hosts(
        cls, host: str, token: str, timeout: int = 10
    ) -> List[RemnawaveHost]:
        """
        Get all hosts.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: List of hosts
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/hosts",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        hosts_data = data.get("response", [])
        return [RemnawaveHost(**host_data) for host_data in hosts_data]

    @classmethod
    async def create_host(
        cls, host: str, token: str, host_data: RemnawaveHostCreate, timeout: int = 10
    ) -> RemnawaveHost:
        """
        Create a new host.

        :param host: API host URL
        :param token: Authentication token
        :param host_data: Host creation data
        :param timeout: Request timeout in seconds
        :return: Host response
        """
        response = await cls.post(
            url=f"{host.rstrip('/')}/api/hosts",
            data=host_data.dict(by_alias=True, exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveHost(**data.get("response", data))

    @classmethod
    async def get_host(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> RemnawaveHost:
        """
        Get a host by UUID.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: Host UUID
        :param timeout: Request timeout in seconds
        :return: Host response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/hosts/{uuid}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveHost(**data.get("response", data))

    @classmethod
    async def update_host(
        cls,
        host: str,
        token: str,
        uuid: str,
        host_data: RemnawaveHostUpdate,
        timeout: int = 10,
    ) -> RemnawaveHost:
        """
        Update a host by UUID.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: Host UUID
        :param host_data: Host update data
        :param timeout: Request timeout in seconds
        :return: Host response
        """
        response = await cls.put(
            url=f"{host.rstrip('/')}/api/hosts/{uuid}",
            data=host_data.dict(by_alias=True, exclude_none=True),
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveHost(**data.get("response", data))

    @classmethod
    async def delete_host(
        cls, host: str, token: str, uuid: str, timeout: int = 10
    ) -> Dict[str, Any]:
        """
        Delete a host by UUID.

        :param host: API host URL
        :param token: Authentication token
        :param uuid: Host UUID
        :param timeout: Request timeout in seconds
        :return: Response data
        """
        response = await cls.delete(
            url=f"{host.rstrip('/')}/api/hosts/{uuid}",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        return await response.json()

    @classmethod
    async def get_subscription_info(
        cls, host: str, short_uuid: str, timeout: int = 10
    ) -> RemnawaveSubscription:
        """
        Get subscription info by short UUID (public endpoint).

        :param host: API host URL
        :param short_uuid: Short UUID of the user
        :param timeout: Request timeout in seconds
        :return: Subscription response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/sub/{short_uuid}/info",
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveSubscription(**data.get("response", data))

    @classmethod
    async def get_subscription_links(
        cls, host: str, short_uuid: str, client_type: str = "", timeout: int = 10
    ) -> str:
        """
        Get subscription links by short UUID and client type.

        :param host: API host URL
        :param short_uuid: Short UUID of the user
        :param client_type: Client type (stash, singbox, mihomo, json, v2ray-json, clash)
        :param timeout: Request timeout in seconds
        :return: Subscription content
        """
        if client_type:
            url = f"{host.rstrip('/')}/api/sub/{short_uuid}/{client_type}"
        else:
            url = f"{host.rstrip('/')}/api/sub/{short_uuid}"

        response = await cls.get(url=url, timeout=timeout)
        return await response.text()

    @classmethod
    async def get_system_stats(
        cls, host: str, token: str, timeout: int = 10
    ) -> RemnawaveSystemStats:
        """
        Get system statistics.

        :param host: API host URL
        :param token: Authentication token
        :param timeout: Request timeout in seconds
        :return: System stats response
        """
        response = await cls.get(
            url=f"{host.rstrip('/')}/api/system/stats",
            headers=cls._generate_headers(token),
            timeout=timeout,
        )
        data = await response.json()
        return RemnawaveSystemStats(**data.get("response", data))



from typing import List, Optional
from datetime import datetime
from pydantic import BaseModel, Field


class GuardUsageDetailStats(BaseModel):
    """Usage detail statistics"""

    start: Optional[datetime] = Field(None, title="Start")
    end: Optional[datetime] = Field(None, title="End")
    remark: Optional[str] = Field(None, title="Remark")
    step: Optional[int] = Field(None, title="Step")
    usage: int = Field(..., title="Usage")


class GuardCountDetailStats(BaseModel):
    """Count detail statistics"""

    start: Optional[datetime] = Field(None, title="Start")
    end: Optional[datetime] = Field(None, title="End")
    count: int = Field(..., title="Count")


class GuardTopSubDetailStats(BaseModel):
    """Top subscription detail statistics"""

    username: str = Field(..., title="Username")
    is_active: bool = Field(..., title="Is Active")
    usage: int = Field(..., title="Usage")


class GuardExpireSubDetailStats(BaseModel):
    """Expire subscription detail statistics"""

    username: str = Field(..., title="Username")
    is_active: bool = Field(..., title="Is Active")
    expire: int = Field(..., title="Expire")


class GuardStatsResponse(BaseModel):
    """General statistics response"""

    total_subscriptions: int = Field(..., title="Total Subscriptions")
    active_subscriptions: int = Field(..., title="Active Subscriptions")
    inactive_subscriptions: int = Field(..., title="Inactive Subscriptions")
    online_subscriptions: int = Field(..., title="Online Subscriptions")
    most_usage_subscription: Optional[str] = Field(
        None, title="Most Usage Subscription"
    )
    most_usage_subscriptions: List[GuardUsageDetailStats] = Field(
        ..., title="Most Usage Subscriptions"
    )
    total_admins: int = Field(..., title="Total Admins")
    active_admins: int = Field(..., title="Active Admins")
    inactive_admins: int = Field(..., title="Inactive Admins")
    most_usage_admins: List[GuardUsageDetailStats] = Field(
        ..., title="Most Usage Admins"
    )
    total_nodes: int = Field(..., title="Total Nodes")
    active_nodes: int = Field(..., title="Active Nodes")
    inactive_nodes: int = Field(..., title="Inactive Nodes")
    most_usage_nodes: List[GuardUsageDetailStats] = Field(..., title="Most Usage Nodes")
    total_lifetime_usages: int = Field(..., title="Total Lifetime Usages")
    total_day_usages: int = Field(..., title="Total Day Usages")
    total_week_usages: int = Field(..., title="Total Week Usages")
    last_24h_usages: List[GuardUsageDetailStats] = Field(..., title="Last 24H Usages")
    last_7d_usages: List[GuardUsageDetailStats] = Field(..., title="Last 7D Usages")


class AdminStatsResponseNew(BaseModel):
    """Combined admin statistics response"""

    usage_limit: Optional[int] = Field(..., title="Usage Limit")
    current_usage: int = Field(..., title="Current Usage")
    left_usage: Optional[int] = Field(..., title="Left Usage")
    lifetime_usage: int = Field(..., title="Lifetime Usage")
    current_day_usage: int = Field(..., title="Current Day Usage")
    current_week_usage: int = Field(..., title="Current Week Usage")
    yesterday_usage: int = Field(..., title="Yesterday Usage")
    last_week_usage: int = Field(..., title="Last Week Usage")
    last_24h_usages: List[GuardUsageDetailStats] = Field(..., title="Last 24H Usages")
    last_7d_usages: List[GuardUsageDetailStats] = Field(..., title="Last 7D Usages")
    last_1m_usages: List[GuardUsageDetailStats] = Field(..., title="Last 1M Usages")
    last_3m_usages: List[GuardUsageDetailStats] = Field(..., title="Last 3M Usages")
    last_1y_usages: List[GuardUsageDetailStats] = Field(..., title="Last 1Y Usages")
    today_top_10_usage_subscriptions: List[GuardTopSubDetailStats] = Field(
        ..., title="Today Top 10 Usage Subscriptions"
    )
    week_top_10_usage_subscriptions: List[GuardTopSubDetailStats] = Field(
        ..., title="Week Top 10 Usage Subscriptions"
    )
    month_top_10_usage_subscriptions: List[GuardTopSubDetailStats] = Field(
        ..., title="Month Top 10 Usage Subscriptions"
    )
    last_24h_counts: List[GuardCountDetailStats] = Field(..., title="Last 24H Counts")
    last_7d_counts: List[GuardCountDetailStats] = Field(..., title="Last 7D Counts")
    last_1m_counts: List[GuardCountDetailStats] = Field(..., title="Last 1M Counts")
    last_3m_counts: List[GuardCountDetailStats] = Field(..., title="Last 3M Counts")
    last_1y_counts: List[GuardCountDetailStats] = Field(..., title="Last 1Y Counts")
    limit_count: int = Field(..., title="Limit Count")
    current_count: int = Field(..., title="Current Count")
    left_count: int = Field(..., title="Left Count")
    total_subscriptions: int = Field(..., title="Total Subscriptions")
    active_subscriptions: int = Field(..., title="Active Subscriptions")
    inactive_subscriptions: int = Field(..., title="Inactive Subscriptions")
    disabled_subscriptions: int = Field(..., title="Disabled Subscriptions")
    expired_subscriptions: int = Field(..., title="Expired Subscriptions")
    limited_subscriptions: int = Field(..., title="Limited Subscriptions")
    today_new_subscriptions: int = Field(..., title="Today New Subscriptions")
    yesterday_new_subscriptions: int = Field(..., title="Yesterday New Subscriptions")
    today_requested_subscriptions: int = Field(
        ..., title="Today Requested Subscriptions"
    )
    today_revoked_subscriptions: int = Field(..., title="Today Revoked Subscriptions")
    today_reseted_subscriptions: int = Field(..., title="Today Reseted Subscriptions")
    today_expire_soon_subscriptions: List[GuardExpireSubDetailStats] = Field(
        ..., title="Today Expire Soon Subscriptions"
    )
    week_expire_soon_subscriptions: List[GuardExpireSubDetailStats] = Field(
        ..., title="Week Expire Soon Subscriptions"
    )
    today_removed_subscriptions: int = Field(..., title="Today Removed Subscriptions")
    yesterday_removed_subscriptions: int = Field(
        ..., title="Yesterday Removed Subscriptions"
    )
    total_removed_subscriptions: int = Field(..., title="Total Removed Subscriptions")



from typing import Optional, Dict, List
from pydantic import BaseModel, Field, constr
from .user import MarzbanProxyTypes


class MarzbanUserTemplateCreate(BaseModel):
    """Schema for creating a user template"""

    name: Optional[str] = Field(None, title="Name")
    data_limit: Optional[int] = Field(
        None, title="Data Limit", description="In bytes, 0 or greater", ge=0
    )
    expire_duration: Optional[int] = Field(
        None, title="Expire Duration", description="In seconds, 0 or greater", ge=0
    )
    username_prefix: Optional[constr(min_length=1, max_length=20)] = Field(
        None, title="Username Prefix"
    )
    username_suffix: Optional[constr(min_length=1, max_length=20)] = Field(
        None, title="Username Suffix"
    )
    inbounds: Dict[MarzbanProxyTypes, List[str]] = Field(
        default_factory=dict, title="Inbounds"
    )


class MarzbanUserTemplateModify(BaseModel):
    """Schema for modifying a user template"""

    name: Optional[str] = Field(None, title="Name")
    data_limit: Optional[int] = Field(None, title="Data Limit", ge=0)
    expire_duration: Optional[int] = Field(None, title="Expire Duration", ge=0)
    username_prefix: Optional[constr(min_length=1, max_length=20)] = Field(
        None, title="Username Prefix"
    )
    username_suffix: Optional[constr(min_length=1, max_length=20)] = Field(
        None, title="Username Suffix"
    )
    inbounds: Dict[MarzbanProxyTypes, List[str]] = Field(
        default_factory=dict, title="Inbounds"
    )


class MarzbanUserTemplateResponse(BaseModel):
    """User template response schema"""

    id: int = Field(..., title="Id")
    name: Optional[str] = Field(None, title="Name")
    data_limit: Optional[int] = Field(None, title="Data Limit", ge=0)
    expire_duration: Optional[int] = Field(None, title="Expire Duration", ge=0)
    username_prefix: Optional[constr(min_length=1, max_length=20)] = Field(
        None, title="Username Prefix"
    )
    username_suffix: Optional[constr(min_length=1, max_length=20)] = Field(
        None, title="Username Suffix"
    )
    inbounds: Dict[MarzbanProxyTypes, List[str]] = Field(
        default_factory=dict, title="Inbounds"
    )



from typing import Optional, List, Dict
from enum import Enum
from pydantic import BaseModel, Field


class MarzneshinProxyTypes(str, Enum):
    """Proxy types enumeration"""

    VMESS = "vmess"
    VLESS = "vless"
    TROJAN = "trojan"
    SHADOWSOCKS = "shadowsocks"
    SHADOWSOCKS2022 = "shadowsocks2022"
    HYSTERIA2 = "hysteria2"
    WIREGUARD = "wireguard"
    TUIC = "tuic"
    SHADOWTLS = "shadowtls"


class MarzneshinProxyHostALPN(str, Enum):
    """Proxy host ALPN enumeration"""

    NONE = "none"
    H2 = "h2"
    HTTP_1_1 = "http/1.1"
    H2_HTTP_1_1 = "h2,http/1.1"
    H3 = "h3"
    H3_H2 = "h3,h2"
    H3_H2_HTTP_1_1 = "h3,h2,http/1.1"


class MarzneshinProxyHostFingerprint(str, Enum):
    """Proxy host fingerprint enumeration"""

    EMPTY = ""
    CHROME = "chrome"
    FIREFOX = "firefox"
    SAFARI = "safari"
    IOS = "ios"
    ANDROID = "android"
    EDGE = "edge"
    T360 = "360"
    QQ = "qq"
    RANDOM = "random"
    RANDOMIZED = "randomized"


class MarzneshinInboundHostSecurity(str, Enum):
    """Inbound host security enumeration"""

    INBOUND_DEFAULT = "inbound_default"
    NONE = "none"
    TLS = "tls"


class MarzneshinFragmentSettings(BaseModel):
    """Fragment settings"""

    packets: str = Field(..., pattern=r"^(:?tlshello|[\d-]{1,32})$", title="Packets")
    length: str = Field(..., pattern=r"^[\d-]{1,32}$", title="Length")
    interval: str = Field(..., pattern=r"^[\d-]{1,32}$", title="Interval")


class MarzneshinXrayNoise(BaseModel):
    """Xray noise settings"""

    type: str = Field(..., pattern=r"^(:?rand|str|base64)$", title="Type")
    packet: str = Field(..., title="Packet")
    delay: str = Field(..., pattern=r"^\d{1,10}(-\d{1,10})?$", title="Delay")


class MarzneshinXMuxSettings(BaseModel):
    """XMux settings"""

    max_concurrency: Optional[str] = Field(
        None, pattern=r"^\d{1,10}(-\d{1,10})?$", title="Max Concurrency"
    )
    max_connections: Optional[str] = Field(
        None, pattern=r"^\d{1,10}(-\d{1,10})?$", title="Max Connections"
    )
    max_reuse_times: Optional[str] = Field(
        None, pattern=r"^\d{1,10}(-\d{1,10})?$", title="Max Reuse Times"
    )
    max_lifetime: Optional[str] = Field(
        None, pattern=r"^\d{1,10}(-\d{1,10})?$", title="Max Lifetime"
    )
    max_request_times: Optional[str] = Field(None, title="Max Request Times")
    keep_alive_period: Optional[int] = Field(None, title="Keep Alive Period")


class MarzneshinSplitHttpSettings(BaseModel):
    """Split HTTP settings"""

    mode: Optional[str] = Field(None, title="Mode")
    no_grpc_header: Optional[bool] = Field(None, title="No Grpc Header")
    padding_bytes: Optional[str] = Field(None, title="Padding Bytes")
    xmux: Optional[MarzneshinXMuxSettings] = Field(None, title="XMux")


class MarzneshinSingBoxMuxSettings(BaseModel):
    """Sing-Box mux settings"""

    max_connections: Optional[int] = Field(None, title="Max Connections")
    max_streams: Optional[int] = Field(None, title="Max Streams")
    min_streams: Optional[int] = Field(None, title="Min Streams")
    padding: Optional[bool] = Field(None, title="Padding")


class MarzneshinMuxCoolSettings(BaseModel):
    """Mux cool settings"""

    concurrency: Optional[int] = Field(None, title="Concurrency")
    xudp_concurrency: Optional[int] = Field(None, title="Xudp Concurrency")
    xudp_proxy_443: Optional[str] = Field(None, title="Xudp Proxy 443")


class MarzneshinMuxSettings(BaseModel):
    """Mux settings"""

    protocol: str = Field(..., title="Protocol")
    sing_box_mux_settings: Optional[MarzneshinSingBoxMuxSettings] = Field(
        None, title="Sing Box Mux Settings"
    )
    mux_cool_settings: Optional[MarzneshinMuxCoolSettings] = Field(
        None, title="Mux Cool Settings"
    )


class MarzneshinInboundHost(BaseModel):
    """Inbound host"""

    remark: str = Field(..., title="Remark")
    address: str = Field(..., title="Address")
    uuid: Optional[str] = Field(None, title="Uuid")
    password: Optional[str] = Field(None, title="Password")
    protocol: Optional[MarzneshinProxyTypes] = Field(None, title="Protocol")
    network: Optional[str] = Field(None, title="Network")
    port: Optional[int] = Field(None, title="Port")
    sni: Optional[str] = Field(None, title="Sni")
    host: Optional[str] = Field(None, title="Host")
    path: Optional[str] = Field(None, title="Path")
    security: MarzneshinInboundHostSecurity = Field(
        MarzneshinInboundHostSecurity.INBOUND_DEFAULT, title="Security"
    )
    alpn: MarzneshinProxyHostALPN = Field(MarzneshinProxyHostALPN.NONE, title="Alpn")
    fingerprint: MarzneshinProxyHostFingerprint = Field(
        MarzneshinProxyHostFingerprint.EMPTY, title="Fingerprint"
    )
    allowinsecure: Optional[bool] = Field(False, title="Allowinsecure")
    is_disabled: Optional[bool] = Field(False, title="Is Disabled")
    fragment: Optional[MarzneshinFragmentSettings] = Field(None, title="Fragment")
    noise: Optional[List[MarzneshinXrayNoise]] = Field(None, title="Noise")
    http_headers: Optional[Dict[str, str]] = Field(None, title="Http Headers")
    mtu: Optional[int] = Field(None, title="Mtu")
    dns_servers: Optional[str] = Field(None, title="Dns Servers")
    allowed_ips: Optional[str] = Field(None, title="Allowed Ips")
    header_type: Optional[str] = Field(None, title="Header Type")
    reality_public_key: Optional[str] = Field(None, title="Reality Public Key")
    reality_short_ids: Optional[List[str]] = Field(None, title="Reality Short Ids")
    flow: Optional[str] = Field(None, title="Flow")
    shadowtls_version: Optional[int] = Field(None, title="Shadowtls Version")
    shadowsocks_method: Optional[str] = Field(None, title="Shadowsocks Method")
    splithttp_settings: Optional[MarzneshinSplitHttpSettings] = Field(
        None, title="Splithttp Settings"
    )
    early_data: Optional[int] = Field(None, title="Early Data")
    mux_settings: Optional[MarzneshinMuxSettings] = Field(None, title="Mux Settings")
    universal: bool = Field(False, title="Universal")
    service_ids: List[int] = Field(default_factory=list, title="Service Ids")
    weight: int = Field(1, title="Weight")
    inbound_id: Optional[int] = Field(None, title="Inbound Id")
    chain_ids: List[int] = Field(default_factory=list, title="Chain Ids")


class MarzneshinInboundHostResponse(MarzneshinInboundHost):
    """Inbound host response"""

    id: int = Field(..., title="Id")


class MarzneshinNode(BaseModel):
    """Node information"""

    id: int = Field(..., title="Id")
    name: str = Field(..., title="Name")
    address: str = Field(..., title="Address")
    port: int = Field(53042, title="Port")
    connection_backend: str = Field("grpclib", title="Connection Backend")
    usage_coefficient: float = Field(1.0, title="Usage Coefficient")


class MarzneshinInbound(BaseModel):
    """Inbound"""

    id: int = Field(..., title="Id")
    tag: str = Field(..., title="Tag")
    protocol: MarzneshinProxyTypes = Field(..., title="Protocol")
    config: str = Field(..., title="Config")
    node: MarzneshinNode = Field(..., title="Node")
    service_ids: List[int] = Field(..., title="Service Ids")


class MarzneshinBackendConfigFormat(int, Enum):
    """Backend config format enumeration"""

    JSON = 0
    YAML = 1
    TOML = 2


class MarzneshinBackendConfig(BaseModel):
    """Backend config"""

    config: str = Field(..., title="Config")
    format: MarzneshinBackendConfigFormat = Field(..., title="Format")


class MarzneshinBackendStats(BaseModel):
    """Backend stats"""

    running: bool = Field(..., title="Running")



from typing import Optional
from pydantic import BaseModel, Field


class RemnawaveHost(BaseModel):
    """Host response"""

    uuid: str
    remark: str
    address: str
    port: int
    path: Optional[str] = None
    sni: Optional[str] = None
    host: Optional[str] = None
    alpn: Optional[str] = None
    fingerprint: Optional[str] = None
    is_disabled: bool = Field(False, alias="isDisabled")
    is_hidden: bool = Field(False, alias="isHidden")
    tag: Optional[str] = None
    view_position: int = Field(..., alias="viewPosition")
    created_at: str = Field(..., alias="createdAt")
    updated_at: str = Field(..., alias="updatedAt")

    class Config:
        populate_by_name = True


class RemnawaveHostCreate(BaseModel):
    """Host creation data"""

    remark: str = Field(..., min_length=1, max_length=40)
    address: str
    port: int = Field(..., ge=1, le=65535)
    path: Optional[str] = None
    sni: Optional[str] = None
    host: Optional[str] = None
    alpn: Optional[str] = None
    fingerprint: Optional[str] = None
    is_hidden: bool = Field(False, alias="isHidden")
    tag: Optional[str] = Field(None, max_length=32)

    class Config:
        populate_by_name = True


class RemnawaveHostUpdate(BaseModel):
    """Host update data"""

    remark: Optional[str] = Field(None, max_length=40)
    address: Optional[str] = None
    port: Optional[int] = Field(None, ge=1, le=65535)
    path: Optional[str] = None
    sni: Optional[str] = None
    host: Optional[str] = None
    alpn: Optional[str] = None
    fingerprint: Optional[str] = None
    is_hidden: Optional[bool] = Field(None, alias="isHidden")
    tag: Optional[str] = Field(None, max_length=32)

    class Config:
        populate_by_name = True



from typing import List, Optional
from datetime import datetime
from enum import Enum
from pydantic import BaseModel, Field


class GuardAdminRole(str, Enum):
    """Admin role types"""

    OWNER = "owner"
    SELLER = "seller"
    RESELLER = "reseller"


class GuardAdminPlaceHolderCategory(str, Enum):
    """Admin placeholder categories"""

    INFO = "info"
    LIMITED = "limited"
    EXPIRED = "expired"
    DISABLED = "disabled"


class GuardAdminPlaceHolder(BaseModel):
    """Admin placeholder configuration"""

    remark: str = Field(..., title="Remark")
    categories: List[GuardAdminPlaceHolderCategory] = Field(..., title="Categories")


class GuardAdminCreate(BaseModel):
    """Schema for creating a new admin"""

    username: str = Field(..., title="Username")
    password: str = Field(..., title="Password")
    role: GuardAdminRole = Field(..., title="Role")
    service_ids: List[int] = Field(..., title="Service Ids")
    create_access: Optional[bool] = Field(False, title="Create Access")
    update_access: Optional[bool] = Field(False, title="Update Access")
    remove_access: Optional[bool] = Field(False, title="Remove Access")
    count_limit: Optional[int] = Field(None, title="Count Limit")
    usage_limit: Optional[int] = Field(None, title="Usage Limit")
    access_prefix: Optional[str] = Field(None, title="Access Prefix")


class GuardAdminUpdate(BaseModel):
    """Schema for updating an existing admin"""

    password: Optional[str] = Field(None, title="Password")
    create_access: Optional[bool] = Field(None, title="Create Access")
    update_access: Optional[bool] = Field(None, title="Update Access")
    remove_access: Optional[bool] = Field(None, title="Remove Access")
    count_limit: Optional[int] = Field(None, title="Count Limit")
    usage_limit: Optional[int] = Field(None, title="Usage Limit")
    service_ids: Optional[List[int]] = Field(None, title="Service Ids")
    placeholders: Optional[List[GuardAdminPlaceHolder]] = Field(
        None, title="Placeholders"
    )
    max_links: Optional[int] = Field(None, title="Max Links")
    shuffle_links: Optional[bool] = Field(None, title="Shuffle Links")
    access_prefix: Optional[str] = Field(None, title="Access Prefix")
    access_title: Optional[str] = Field(None, title="Access Title")
    access_description: Optional[str] = Field(None, title="Access Description")


class GuardAdminCurrentUpdate(BaseModel):
    """Schema for updating current admin"""

    password: Optional[str] = Field(None, title="Password")
    placeholders: Optional[List[GuardAdminPlaceHolder]] = Field(
        None, title="Placeholders"
    )
    max_links: Optional[int] = Field(None, title="Max Links")
    shuffle_links: Optional[bool] = Field(None, title="Shuffle Links")
    access_title: Optional[str] = Field(None, title="Access Title")
    access_description: Optional[str] = Field(None, title="Access Description")


class GuardAdminResponse(BaseModel):
    """Admin response schema"""

    id: int = Field(..., title="Id")
    enabled: bool = Field(..., title="Enabled")
    username: str = Field(..., title="Username")
    role: GuardAdminRole = Field(..., title="Role")
    service_ids: List[int] = Field(..., title="Service Ids")
    create_access: Optional[bool] = Field(..., title="Create Access")
    update_access: Optional[bool] = Field(..., title="Update Access")
    remove_access: Optional[bool] = Field(..., title="Remove Access")
    count_limit: Optional[int] = Field(..., title="Count Limit")
    current_count: Optional[int] = Field(..., title="Current Count")
    left_count: Optional[int] = Field(..., title="Left Count")
    reached_count_limit: Optional[bool] = Field(..., title="Reached Count Limit")
    usage_limit: Optional[int] = Field(..., title="Usage Limit")
    current_usage: Optional[int] = Field(..., title="Current Usage")
    left_usage: Optional[int] = Field(..., title="Left Usage")
    reached_usage_limit: Optional[bool] = Field(..., title="Reached Usage Limit")
    placeholders: Optional[List[GuardAdminPlaceHolder]] = Field(
        ..., title="Placeholders"
    )
    max_links: Optional[int] = Field(..., title="Max Links")
    shuffle_links: Optional[bool] = Field(..., title="Shuffle Links")
    api_key: str = Field(..., title="Api Key")
    access_title: Optional[str] = Field(None, title="Access Title")
    access_description: Optional[str] = Field(None, title="Access Description")
    last_login_at: Optional[datetime] = Field(..., title="Last Login At")
    last_online_at: Optional[datetime] = Field(..., title="Last Online At")
    created_at: datetime = Field(..., title="Created At")
    updated_at: datetime = Field(..., title="Updated At")


class GuardAdminToken(BaseModel):
    """Admin authentication token"""

    access_token: str = Field(..., title="Access Token")
    token_type: str = Field("bearer", title="Token Type")


class GuardAdminUsageLog(BaseModel):
    """Admin usage log entry"""

    usage: int = Field(..., title="Usage")
    created_at: datetime = Field(..., title="Created At")


class GuardAdminUsageLogsResponse(BaseModel):
    """Admin usage logs response"""

    admin: GuardAdminResponse = Field(..., title="Admin")
    usages: List[GuardAdminUsageLog] = Field(..., title="Usages")



from .admin import (
    RemnawaveAdmin,
    RemnawaveAdminCreate,
    RemnawaveAdminUpdate,
    RemnawaveToken,
)
from .user import (
    RemnawaveUser,
    RemnawaveUserCreate,
    RemnawaveUserUpdate,
    RemnawaveUserStatus,
)
from .node import (
    RemnawaveNode,
    RemnawaveNodeCreate,
    RemnawaveNodeUpdate,
)
from .host import (
    RemnawaveHost,
    RemnawaveHostCreate,
    RemnawaveHostUpdate,
)
from .subscription import RemnawaveSubscription
from .system import RemnawaveSystemStats

__all__ = [
    "RemnawaveAdmin",
    "RemnawaveAdminCreate",
    "RemnawaveAdminUpdate",
    "RemnawaveToken",
    "RemnawaveUser",
    "RemnawaveUserCreate",
    "RemnawaveUserUpdate",
    "RemnawaveUserStatus",
    "RemnawaveNode",
    "RemnawaveNodeCreate",
    "RemnawaveNodeUpdate",
    "RemnawaveHost",
    "RemnawaveHostCreate",
    "RemnawaveHostUpdate",
    "RemnawaveSubscription",
    "RemnawaveSystemStats",
]



from typing import Optional, List
from pydantic import BaseModel, Field


class MarzneshinAdminsStats(BaseModel):
    """Admins statistics"""

    total: int = Field(..., title="Total")


class MarzneshinNodesStats(BaseModel):
    """Nodes statistics"""

    total: int = Field(..., title="Total")
    healthy: int = Field(..., title="Healthy")
    unhealthy: int = Field(..., title="Unhealthy")


class MarzneshinUsersStats(BaseModel):
    """Users statistics"""

    total: int = Field(..., title="Total")
    active: int = Field(..., title="Active")
    on_hold: int = Field(..., title="On Hold")
    expired: int = Field(..., title="Expired")
    limited: int = Field(..., title="Limited")
    online: int = Field(..., title="Online")


class MarzneshinTrafficUsageSeries(BaseModel):
    """Traffic usage series"""

    step: int = Field(3600, title="Step")
    total: int = Field(0, title="Total")
    usages: List[List[int]] = Field(..., title="Usages")


class MarzneshinTelegramSettings(BaseModel):
    """Telegram settings"""

    token: str = Field(..., title="Token")
    admin_id: List[int] = Field(..., title="Admin Id")
    channel_id: Optional[int] = Field(None, title="Channel Id")
